--  artregion.lua

--[[

DrawRegion with artwork

--]]

require("base.drawregion")
require("base.rectangle")

ArtRegion =    class(DrawRegion,
                        function(
                            a, name,
                            frameWidth, frameHeight, 
                            frameLeftXpos, frameTopYpos,
                            boundWidth, boundHeight,
                            boundLeftXpos, boundTopYpos, 
                            isVisible,

                            artID,
                            maxDimSize,
                            fallbackID
                           )

                            DrawRegion.init(
                                a, name,
                                frameWidth, frameHeight, 
                                frameLeftXpos, frameTopYpos,
                                boundWidth, boundHeight,
                                boundLeftXpos, boundTopYpos, 
                                isVisible)

                            a.artID =           artID
                            a.maxDimSize =      maxDimSize
                            a.fallbackID =      fallbackID
                            a.imageSize = {}
                            a.artSurface = nil
                        end)

function ArtRegion:Draw()

--  Draws artwork centered within (or clipped to) bounds

--  Acquire image; also acquire fallback image (by id)

    local   fallbackWidth, fallbackHeight, offsetXpos, offsetYpos = 
                0, 0, 0, 0

    if  self.artID ~= nil then
   
    if not (type(self.artID) == "string") then
        self.artSurface = Display.AcquireImage(self.artID, self.maxDimSize)

        --Log("Got desired ID " .. self.fallbackID)
    else
        if self.artID == "" then
            return
        end

        self.artSurface =
            Display.AcquireImageUri(
            self.artID, 
            self.maxDimSize)
        
        if self.artSurface == nil then

            if self.fallbackID == -1 then
                return
            end
            
        --  acquire fallback image by ID 
        --  while ImageManager deals with URI

            self.artSurface =
                Display.AcquireImage(
                    self.fallbackID, 
                    self.maxDimSize)

            --Log("Got fallback ID " .. self.fallbackID)
        else
            --Log("Got desired URI " .. self.fallbackID)
        end
    end

    end -- if  self.artID ~= nil 

    if  self.artSurface ~= nil then    
    self.imageSize.width, self.imageSize.height =  
        self.artSurface:getSize()


--    self:PrintArtworkSize()    

--  Calculate offsets

    offsetXpos = (self.boundWidth/2) - math.floor(self.imageSize.width/2)
    offsetYpos =  (self.boundHeight/2) - math.floor(self.imageSize.height/2)

--  Create SDL rectangles for clipping and bounds

    local   imageClippingRect = 
                SDL.newRect(
                0, 0,
                self.imageSize.width, self.imageSize.height)

    local   imageBoundRect = 
                SDL.newRect(
                self.boundLeftXpos + offsetXpos, 
                self.boundTopYpos + offsetYpos,
                self.boundWidth, self.boundHeight)

            --print("BLIT AT: ", self.boundLeftXpos, ", ", self.boundTopYpos)
--  Blit image to screen

    SDL.BlitSurface(
        self.artSurface, imageClippingRect, screen, imageBoundRect)            
    end --   if  self.artSurface ~= nil
end


function ArtRegion:OnExit()

end

function
    ArtRegion:DrawRuleLine(color)

--  Draws inner (black) and outer (color) rule lines around region

if  self.artID ~= nil then

    local   offsetXpos, offsetYpos = 0, 0
    local   isFilled = false

--  Calculate offsets

    offsetXpos = (self.boundWidth/2) - math.floor(self.imageSize.width/2)
    offsetYpos =  (self.boundHeight/2) - math.floor(self.imageSize.height/2)


--[[
        Display.DrawRectangleFilled(
            self.frameLeftXpos,
            self.frameTopYpos,
            self.frameWidth,
            self.frameHeight,
            self.color)

    else

        Display.DrawRectangle(
            self.frameLeftXpos,
            self.frameTopYpos,
            self.frameWidth,
            self.frameHeight,
            self.color)
--]]

    local ruleRect = Rectangle("ruleRect", self.imageSize.width - 1, self.imageSize.height - 1, 
                        self.boundLeftXpos + offsetXpos, 
                        self.boundTopYpos + offsetYpos, 
                        color, isFilled)
    local ruleRect2 = Rectangle("ruleRect", self.imageSize.width - 3, self.imageSize.height - 3, 
                        self.boundLeftXpos + offsetXpos + 1, 
                        self.boundTopYpos + offsetYpos + 1, 
                        black, isFilled)

    ruleRect:Draw()
    ruleRect2:Draw()

--[[
    Display.DrawRectangle(self.boundLeftXpos + offsetXpos, 
                        self.boundTopYpos + offsetYpos,
                        self.imageSize.width - 1, self.imageSize.height - 1,
                        color)

    Display.DrawRectangle(self.boundLeftXpos + offsetXpos + 1, 
                        self.boundTopYpos + offsetYpos + 1,
                        self.imageSize.width - 3, self.imageSize.height - 3,
                        black)
--]]

--[[
--  TEST - draw eight lines

--  top inner
    Display.DrawLine(  self.boundLeftXpos + offsetXpos + 1,
                        self.boundTopYpos + offsetYpos + 1,
                        self.boundLeftXpos + offsetXpos + 1 + self.imageSize.width - 3,
                        self.boundTopYpos + offsetYpos + 1,
                        black)
--  bottom inner
    Display.DrawLine(  self.boundLeftXpos + offsetXpos + 1,
                        self.boundTopYpos + offsetYpos + 1 + self.imageSize.height - 3,
                        self.boundLeftXpos + offsetXpos + 1 + self.imageSize.width - 3,
                        self.boundTopYpos + offsetYpos + 1 + self.imageSize.height - 3,
                        black)

--  left inner
    Display.DrawLine(  self.boundLeftXpos + offsetXpos + 1,
                        self.boundTopYpos + offsetYpos + 2,
                        self.boundLeftXpos + offsetXpos + 1,
                        self.boundTopYpos + offsetYpos + 1 + self.imageSize.height - 4,
                        black)

--  right inner
    Display.DrawLine(  self.boundLeftXpos + offsetXpos + 1 + self.imageSize.width - 3,
                        self.boundTopYpos + offsetYpos + 2,
                        self.boundLeftXpos + offsetXpos + 1 + self.imageSize.width - 3,
                        self.boundTopYpos + offsetYpos + 1 + self.imageSize.height - 4,
                        black)

--  top outer
    Display.DrawLine(  self.boundLeftXpos + offsetXpos,
                        self.boundTopYpos + offsetYpos,
                        self.boundLeftXpos + offsetXpos + self.imageSize.width - 1,
                        self.boundTopYpos + offsetYpos,
                        color)
--  bottom outer
    Display.DrawLine(  self.boundLeftXpos + offsetXpos,
                        self.boundTopYpos + offsetYpos + self.imageSize.height - 1,
                        self.boundLeftXpos + offsetXpos + self.imageSize.width - 1,
                        self.boundTopYpos + offsetYpos + self.imageSize.height - 1,
                        color)

--  left outer
    Display.DrawLine(  self.boundLeftXpos + offsetXpos,
                        self.boundTopYpos + offsetYpos + 1,
                        self.boundLeftXpos + offsetXpos,
                        self.boundTopYpos + offsetYpos + self.imageSize.height - 2,
                        color)

--  right outer
    Display.DrawLine(  self.boundLeftXpos + offsetXpos + self.imageSize.width - 1,
                        self.boundTopYpos + offsetYpos + 1,
                        self.boundLeftXpos + offsetXpos + self.imageSize.width - 1,
                        self.boundTopYpos + offsetYpos + self.imageSize.height - 2,
                        color)
--]]
end
    end

function 
    ArtRegion:PrintArtworkSize()

        print("Artwork width = " .. self.imageSize.width)
        print("Artwork height = " .. self.imageSize.height)

    end
