--[[

DrawRegion      Base class for visible, bounded Region 
                using discrete Cartesian coordinates

                Adds concept of frame and bound(s)
                to Region

                Content is drawn within bound(s) e-f-g-h

                Bound(s) may exceed region, if required

                A-----------------------B
                |         tBpad         |
                |      e*********f      |
                |      *         *      |
                |lBpad *         * rBpad|
                |      *         *      |
                |      h*********g      | g =   boundRightXpos, 
                |         bBpad         |       boundBottomYpos 
                D-----------------------C C =   frameRightXpos,  
                                                frameBottomYpos

--]]

--require( "class" )
require( "base.region" )
require( "base.rectangle" )

DrawRegion =    class( Region, 
                    function( 
                        a, name,
                        frameWidth, frameHeight, 
                        frameLeftXpos, frameTopYpos,
                        isVisible )

                        Region.init( 
                            a, name, 
                            frameWidth, frameHeight, 
                            frameLeftXpos, frameTopYpos)

                        a.isVisible = isVisible

                        a.forceRedraw = forceRedraw --  not parameterized
                    end )

function DrawRegion:OnEnter()
    self.forceRedraw = true        
end

function DrawRegion:BottomRightBounds()  --  returns point 'g' as shown above
    local boundRightXpos = math.floor(self.frameLeftXpos + self.frameWidth - 1 )
    local boundBottomYpos = math.floor(self.frameTopYpos + self.frameHeight - 1 )

    return boundRightXpos, boundBottomYpos
end

function DrawRegion:DrawFrameAndBoundsRects()
--  Frame in red
    debugDrawRegionFrame = Rectangle(
                                "debugDrawRegionFrame",
                                self.frameWidth-1, self.frameHeight-1,
                                self.frameLeftXpos, self.frameTopYpos,
                                red, false,
                                true )
--  Bounds in blue
    debugDrawRegionBounds = Rectangle(
                                "debugDrawRegionBounds",    
                                self.boundWidth-1, self.boundHeight-1,
                                self.boundLeftXpos, self.boundTopYpos,
                                blue, false,
                                true )

    debugDrawRegionFrame:Draw()
    debugDrawRegionBounds:Draw()
end 

function DrawRegion:WipeFrameFillBlack()
--  Wipes contents of frame with black

    local   isFilled = true
    local   fillColor = black 

    local artWipeFrameFillBlack = Rectangle( "artWipeFrameFillBlack",
                            self.frameWidth - 1, 
                            self.frameHeight - 1,
                            self.frameLeftXpos, 
                            self.frameTopYpos,
                            fillColor, isFilled,
                            self.isVisible )

    artWipeFrameFillBlack:Draw()

    local drawn = true
    return drawn

    end

function DrawRegion:WipeFrameFill(fillColor)

--  Wipes contents of frame with black

    local   isFilled = true

    local artWipeFrameFillBlack = Rectangle( "artWipeFrameFillBlack",
                            self.frameWidth - 1, 
                            self.frameHeight - 1,
                            self.frameLeftXpos, 
                            self.frameTopYpos,
                            fillColor, isFilled,
                            self.isVisible )

    artWipeFrameFillBlack:Draw()

    local drawn = true
    return drawn

    end

function DrawRegion:DrawDebugLabel()

    local drawn = false

--  stuff goes here TODO

    local   font = 23
    local   debugLabel = self.name .. " TODO"

    local   debugLabel = 
                Display.RenderText(font, debugLabel)
    local   textWidth, textHeight = debugLabel:getSize()

    --  Align text V & H

    local rect = SDL.newRect(
        self.frameLeftXpos + padding, 
        self.frameTopYpos + padding, 
        textWidth, 
        textHeight)    

    SDL.BlitSurface(debugLabel, nil, screen, rect)

    drawn = true
    return  drawn

    end

function DrawRegion:OnExit()
    self:WipeFrameFillBlack()    
end
