--  Region      Base class for discrete Cartesian coordinates

--[[

    Ti------An########      T = TopLeft()
    |       |#       #      C = Center()
    |   C   |#       #      B = BottomRight()
    |       |#       #
    D-------B#########      n = frameLeftXpos + frameWidth, frameTopYpos
             k*******#      k = frameLeftXpos + frameWidth, frameTopYpos + frameHeight
             *       *
             *       *      i to A to B to D to T -> framePerim
                            Area includes all pixels on or between
                                points T, A, B, D
--]]

require( "class" )

Region =        class( --   NO PARENT
                    function( 
                        a, name,
                        frameWidth, frameHeight, 
                        frameLeftXpos, frameTopYpos )

                        a.name = name
                        a.frameWidth = frameWidth
                        a.frameHeight = frameHeight
                        a.frameLeftXpos = frameLeftXpos
                        a.frameTopYpos = frameTopYpos
                    end ) 

function
    Region:TopLeft()

    return          self.frameLeftXpos, self.frameTopYpos
end

function
    Region:BottomRight()        --  returns point 'B' as shown above
        frameRightXpos =     math.floor(
                        self.frameLeftXpos + self.frameWidth - 1 )
        frameBottomYpos =    math.floor(
                        self.frameTopYpos + self.frameHeight - 1 )

    return          frameRightXpos, frameBottomYpos
end

function
    Region:Center()             --  returns point 'C' as shown above
        frameCenterXpos =    math.floor(
                        (self.frameLeftXpos + self.frameWidth) / 2 )
        frameCenterYpos =    math.floor(
                        (self.frameTopYpos + self.frameHeight) / 2 )
    
    return          frameCenterXpos, frameCenterYpos
end

function
    Region:Perimeter()
        framePerim =         (self.frameWidth - 1 + self.frameHeight - 1 ) * 2

    return          framePerim
end

function
    Region:Area()
        area =          self.frameWidth * self.frameHeight

    return          area
end

function
    Region:PrintBounds()

    rX, bY = self:BottomRight()

    print(  "Region " .. 
                self.name ..
            " is " ..
                self.frameWidth ..
            " px w x " ..
                self.frameHeight ..
            " px h, from (" ..
                self.frameLeftXpos .. 
            ", " .. 
                self.frameTopYpos ..
            ") to (" ..
                rX .. 
            ", " .. 
                bY ..
            ")"
            )

end 
