--  command.lua

require("command.DIRECTION_UP")
require("command.DIRECTION_DOWN")
require("command.ENTER")
require("command.ESCAPE")
require("command.OPTION")

require("command.VOLUME_UP")
require("command.VOLUME_DOWN")
require("command.MUTE")
require("command.HOME")
require("command.POWER")

require("command.PLAYPAUSE")
require("command.STOP")
require("command.PREVIOUS")
require("command.NEXT")
require("command.INPUT")
require("command.BROWSE")
require("command.HELD_L4")
require("command.HELD_L5")
require("command.HELD_R3")
require("command.HELD_R4")

require("command.REMOTE_UP")
require("command.REMOTE_DOWN")
require("command.REMOTE_SELECT")
require("command.REMOTE_BACK")
require("command.REMOTE_RIGHT")
require("command.REMOTE_LEFT")

require("command.DELETE_ALL")
require("command.REMOTE_ADDPLUS")

require("command.SEEK")

require("format.currentsource")

require("context")

function GetInputType(display)
    local source_index = display:GetSourceIndex()   
    local source_list  = display:GetSourceList()

    sourceName = ""
    sourceType = ""

    if  source_list ~= nil then
        local source = CurrentSource(source_list, source_index)
        if  source ~= nil then
            sourceName = source.name
            sourceType = source.type
        else
            sourceName = ""
            sourceType = ""
        end
    end
end


--  Front-panel buttons (left) -----------------------------------------
--  These are largely "soft" buttons

-- Up arrow keyboard event
function OnL1()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        DIRECTION_UP.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

-- Down arrow keyboard event
function OnL2()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        DIRECTION_DOWN.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

-- Right arrow keyboard event
function OnL3()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        ENTER.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

-- Left arrow keyboard event
function OnL4()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        ESCAPE.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

-- o keyboard event (not zero)
function OnL5()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then
        OPTION.ProcessContext(upnp, display)

    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  Front-panel buttons (right) ----------------------------------------

--  a keyboard event
function OnR1()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        VOLUME_UP.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  z keyboard event
function OnR2()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        VOLUME_DOWN.ProcessContext(upnp, display) 
    end   

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  m keyboard event
function OnR3()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        MUTE.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  h keyboard event
function OnR4()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        HOME.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  q keyboard event
function OnR5()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        POWER.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

function OnHeldL4()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        HELD_L4.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

function OnHeldL5()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        HELD_L5.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

function OnHeldR3()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        HELD_R3.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

function OnHeldR4()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        HELD_R4.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  Remote control functions
function OnRemoteUp()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        REMOTE_UP.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

function OnRemoteDown()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        REMOTE_DOWN.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

function OnRemoteSelect()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        REMOTE_SELECT.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

function OnRemoteBack()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        REMOTE_BACK.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

function OnRemoteRight()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        REMOTE_RIGHT.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

function OnRemoteLeft()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        REMOTE_LEFT.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  Extended functions / testing ---------------------------------------

--  Spacebar - REQUIRED

function OnPlayPauseToggle()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        PLAYPAUSE.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  P keyboard event
function OnPrev()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        PREVIOUS.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  N keyboard event
function OnNext()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        NEXT.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  Delete

function OnDeleteAll()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        DELETE_ALL.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

function OnRemoteAddPlus()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        REMOTE_ADDPLUS.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  Misc ---------------------------------------------------------------

function OnSeek()
    local upnp =        GetUpnp()
    local display =     GetDisplayData()

    GetInputType(display)

    if upnp:numActionsInProgress() < 1 then

        SEEK.ProcessContext(upnp, display)
    end

    ReleaseDisplayData(display)
    ReleaseUpnp(upnp)
end

--  Initialization functions -------------------------------------------

function InitContextOnStartup()

    local display =     GetDisplayData()
    local context =      display:GetContext()

    GetInputType(display)

    newContext = Contexts.PLAY_INFO

    if display:GetContext() ~= Contexts.USB and display:GetContext() ~= Contexts.UPDATE and display:GetContext() ~= Contexts.NETWORK then
        display:SetContext(newContext)
        Log("Context initialized to PLAY_INFO")
    else
        Log("Context was already initialized to USB or UPDATE or NETWORK")
    end

    ReleaseDisplayData(display)

end

function InitMuteOnStartup()

    mute_state = false 

end

function OnTestCmd()
    -- purposefully does nothing
    --Log("Test command!")
end

--  Command defaults registered at startup -----------------------------

function RegisterCommands()

    InitContextOnStartup()
    InitMuteOnStartup()

--  Front-panel buttons

    CommandSetFunc(Commands.L1, "OnL1")
    CommandSetFunc(Commands.L2, "OnL2")
    CommandSetFunc(Commands.L3, "OnL3")
    CommandSetFunc(Commands.L4, "OnL4")
    CommandSetFunc(Commands.L5, "OnL5")

    CommandSetFunc(Commands.R1, "OnR1")
    CommandSetFunc(Commands.R2, "OnR2")
    CommandSetFunc(Commands.R3, "OnR3")
    CommandSetFunc(Commands.R4, "OnR4")
    CommandSetFunc(Commands.R5, "OnR5")

--  Remote control buttons

    CommandSetFunc(Commands.REMOTE_UP, "OnRemoteUp")
    CommandSetFunc(Commands.REMOTE_DOWN, "OnRemoteDown")
    CommandSetFunc(Commands.REMOTE_SELECT, "OnRemoteSelect")
    CommandSetFunc(Commands.REMOTE_BACK, "OnRemoteBack")
    CommandSetFunc(Commands.REMOTE_LEFT, "OnRemoteLeft")
    CommandSetFunc(Commands.REMOTE_RIGHT, "OnRemoteRight")
    CommandSetFunc(Commands.REMOTE_ADDPLUS, "OnRemoteAddPlus")

--  Extended functions / testing

    CommandSetFunc(Commands.PAUSE, "OnPlayPauseToggle")
    CommandSetFunc(Commands.STOP, "OnMute")
    CommandSetFunc(Commands.PREV, "OnPrev")
    CommandSetFunc(Commands.NEXT, "OnNext")
    CommandSetFunc(Commands.DELETE_ALL, "OnDeleteAll")

    CommandSetFunc(Commands.HELD_L4, "OnHeldL4")
    CommandSetFunc(Commands.HELD_L5, "OnHeldL5")
    CommandSetFunc(Commands.HELD_R3, "OnHeldR3")
    CommandSetFunc(Commands.HELD_R4, "OnHeldR4")

--  Misc
    CommandSetFunc(Commands.SEEK, "OnSeek")

    CommandSetFunc(Commands.TEST_CMD, "OnTestCmd")
    
end
