
require("remote_commands")

function AdjustToLimits(value, limits)
    if value > limits.second then
        return limits.second
    elseif value < limits.first then
        return limits.first
    end
    return value
end

function CmdDummy()
    Log("Dummy command!")
end

function CmdSwitchInput(number)
    Log("CmdSwitchInput: " .. number)
    GetPreampSetting():SetCurrentInput(number - 1)
end

function AdjustHelper(cmd_string, current, change, resolution, limits)
    local new = current + change * resolution
    new = AdjustToLimits(new, limits)
    Log(cmd_string .. ": " .. current .. " " .. change .. " " .. new)
    return new
end

function EncRampHelper(res, value, stage_2, stage_3, stage_4, stage_5_neg, stage_5_pos)
    local res_mult = 1
    if res == 5 then
        res_mult = 2
    elseif res == 1 then
        res_mult = 10
    end
    if math.abs(value) == 1 then
        value = value * 1
    elseif math.abs(value) == 2 then
        value = (value / 2) * stage_2 * res_mult
    elseif math.abs(value) == 3 then
        value = (value / 3) * stage_3 * res_mult
    elseif math.abs(value) == 4 then
        value = (value / 4) * stage_4 * res_mult
    elseif value == 5 then
        value = (value / 5) * stage_5_pos * res_mult
    elseif value == -5 then
        value = (value / 5) * stage_5_neg * res_mult
    else
        value = 0
    end
    return value
end

function CmdEncInputTrim(index, value)
    value = EncRampHelper(GetPreampSetting():GetInputResolution(), value, 1, 2, 4, 20, 4)
    local input = GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput())
    local new = AdjustHelper("CmdEncInputTrim", input.trim, value, GetPreampSetting():GetInputResolution(),
                             GetPreampSetting():GetInputLimits())
    GetPreampSetting():SetInputDevice(input.index, input.name, new, input.balance, input.polarity,
                                      input.theater_mode, input.recordable)
end

function CmdEncBlend(index, value)
    -- no blend resolution, so don't set multiplier
    value = EncRampHelper(10, value, 1, 1, 1, 1, 1)
    local old = GetPreampSetting():GetBlend()
    local new = AdjustHelper("CmdEncBlend", old, value, 1, GetPreampSetting():GetBlendLimits())
    GetPreampSetting():SetBlend(new)
end

function CheckStickyTimer()
    if command_sticky_time ~= nil then
        local now = SDL.GetTicks()
        if now - command_sticky_time < 500 then
            return true
        else
            command_sticky_time = nil
        end
    end
    return false
end

function SetStickyTimer(old, new)
    if new == 0 or (old * new) < 0 then
        command_sticky_time = SDL.GetTicks()
        return true
    end
    return false
end

function CmdEncBalance(index, value)
    if CheckStickyTimer() then
        return
    end
    value = EncRampHelper(GetPreampSetting():GetVolumeResolution(), value, 1, 2, 3, 4, 4)
    local old = GetPreampSetting():GetBalance()
    local new = AdjustHelper("CmdEncBalance", old, value, GetPreampSetting():GetVolumeResolution(),
                             GetPreampSetting():GetBalanceLimits())
    if (old == -201 and value > 0) then
        new = -200
    elseif (old == 201 and value < 0) then
        new = 200
    end
    if SetStickyTimer(old, new) then
        new = 0
    end
    GetPreampSetting():SetBalance(new)
end

function CmdEncMasterVolume(index, value)
    if GetPreampSetting():GetMute() and value > 0 then
        GetPreampSetting():SetMute(false)
    end
    if GetPreampSetting():GetSoft() and value > 0 then
        GetPreampSetting():SetSoft(false)
    end
    value = EncRampHelper(GetPreampSetting():GetVolumeResolution(), value, 1, 2, 4, 20, 4)
    local old = GetPreampSetting():GetMasterVolume()
    local new = AdjustHelper("CmdEncMasterVolume", old, value, GetPreampSetting():GetVolumeResolution(),
                             GetPreampSetting():GetVolumeLimits())
    if old == -1001 and value > 0 then
        new = -1000
    end
    GetPreampSetting():SetMasterVolume(new)
end

function CmdEncOutputVolume(index, value)
    if index > 4 then
        if CheckStickyTimer() then
            return
        end
        value = EncRampHelper(GetPreampSetting():GetOutputResolution(), value, 1, 2, 4, 20, 4)
        local output = GetPreampSetting():GetOutputDevice(index - 5)
        local old = output.volume
        local new = AdjustHelper("CmdEncOutputVolume", old, value, GetPreampSetting():GetOutputResolution(),
                                 GetPreampSetting():GetOutputLimits())
        if SetStickyTimer(old, new) then
            new = 0
        end
        GetPreampSetting():SetOutputDevice(output.name, index - 5, new, output.mute, output.polarity, output.polarity_temp)
    else
        LogLevel(LogLevels.Err, "index " .. index .. " is invalid for output volume encoder")
    end
end

function CmdMute(number)
    Log("CmdMute: " .. number)
    local mute =  GetPreampSetting():GetMute()
    GetPreampSetting():SetMute(not mute)
end

function CmdSoft(number)
    Log("CmdSoft: " .. number)
    local soft = GetPreampSetting():GetSoft()
    GetPreampSetting():SetSoft(not soft)
end

function CmdOutputMute(number)
    Log("CmdOutputMute: " .. number)
    local output = GetPreampSetting():GetOutputDevice(number - 10)
    GetPreampSetting():SetOutputDevice(output.name, number - 10, output.volume, not output.mute, output.polarity, output.polarity_temp)
end

function CmdOutputName(name)
    Log("CmdOutputName: " .. name)
    local output = GetPreampSetting():GetOutputDevice(number - 10)
    GetPreampSetting():SetOutputDevice(name, number - 10, output.volume,  output.mute, output.polarity, output.polarity_temp)
end

function CmdPolarity(number)
    Log("CmdPolarity: " .. number)
    local preamp = GetPreampSetting()
    if number == 10 then
        local output = preamp:GetOutputDevice(number - 10)
        preamp:SetOutputDevice(output.name, number - 10, output.volume, output.mute, output.polarity, not output.polarity_temp)
    elseif number == 11 then
        local output = preamp:GetOutputDevice(number - 10)
        preamp:SetOutputDevice(output.name, number - 10, output.volume, output.mute, output.polarity, not output.polarity_temp)
    elseif number == 12 then
        local output = preamp:GetOutputDevice(number - 10)
        preamp:SetOutputDevice(output.name, number - 10, output.volume, output.mute, output.polarity, not output.polarity_temp)
    elseif number == 14 then
        local pol = preamp:GetPolarity()
        pol = not pol
        preamp:SetPolarity(pol)
    elseif number == 15 then
        local pol = preamp:GetPolarityRight()
        pol = not pol
        preamp:SetPolarityRight(pol)
    end
end

function CmdSwitchButtons(number)
    Log("CmdSwitchButtons: " .. number)
    if number == 13 then
        if main_view_context.icon_region ~= option_icons then
            main_view_context.icon_region = option_icons
            main_view_context.icon_strs = option_icon_strs
            main_view_context.button_commands = option_commands
        else
            main_view_context.icon_region = main_icons
            main_view_context.icon_strs = main_icon_strs
            main_view_context.button_commands = main_commands
        end
    elseif number == 14 then
        if main_view_context.icon_region ~= setup_icons then
            main_view_context.icon_region = setup_icons
            main_view_context.icon_strs = setup_icon_strs
            main_view_context.button_commands = setup_commands
            -- start timer to return to home
            return_home_time = SDL.GetTicks()
        else
            main_view_context.icon_region = main_icons
            main_view_context.icon_strs = main_icon_strs
            main_view_context.button_commands = main_commands
        end
    else
        main_view_context.icon_region = main_icons
        main_view_context.icon_strs = main_icon_strs
        main_view_context.button_commands = main_commands
    end
end

function CmdAuxSelect(number)
    Log("CmdAuxSelect: " .. number)
    main_view_context.icon_region = aux_select_icons
    main_view_context.icon_strs = aux_select_icon_strs
    main_view_context.button_commands = aux_select_commands    
end

function CmdAuxSelectInput(number)
    Log("CmdAuxSelectInput: " .. number)
    local preamp = GetPreampSetting()
    if number < 7 then
        preamp:SetCurrentAux(number - 1)
    end
    
    main_view_context.icon_region = option_icons
    main_view_context.icon_strs = option_icon_strs
    main_view_context.button_commands = option_commands    
end

function CmdAuxDisable(number)
    Log("CmdAuxDisable: " .. number)
    local preamp = GetPreampSetting()
    if preamp:GetAuxDisabled() then
        preamp:SetAuxDisabled(false)
    else
        preamp:SetAuxDisabled(true)
    end
end

function CmdEncVolumeRes(index, value)
    Log("CmdEncVolumeRes: " .. value)
    value = EncRampHelper(10, value, 1, 1, 1, 1, 1)
    local old = GetPreampSetting():GetVolumeResolution()
    local res = old
    if value > 0 then
        if old == 1 then
            res = 5
        elseif old == 5 then
            res = 10
        end
    else
        if old == 5 then
            res = 1
        elseif old == 10 then
            res = 5
        end
    end

    GetPreampSetting():SetVolumeResolution(res)
end

function CmdEncVolumeScale(index, value)
    value = EncRampHelper(10, value, 1, 2, 3, 4, 4)
    local old = GetPreampSetting():GetVolumeScale()
    local limits = {}
    limits.first = current_popup[popup_index][4]
    limits.second = current_popup[popup_index][5]
    local new = AdjustHelper("CmdEncVolumeScale", old, value, 1, limits)
    GetPreampSetting():SetVolumeScale(new)
end

function CmdEncVolumeDefault(index, value)
    value = EncRampHelper(10, value, 1, 2, 3, 4, 4)
    local old = GetPreampSetting():GetDefaultVolume()
    local limits = {}
    limits.first = current_popup[popup_index][4]
    limits.second = current_popup[popup_index][5]
    local new = AdjustHelper("CmdEncVolumeDefault", old, value, 10, limits)
    GetPreampSetting():SetDefaultVolume(new)
end

function CmdEncVolumeMax(index, value)
    value = EncRampHelper(10, value, 1, 2, 3, 4, 4)
    local old = GetPreampSetting():GetMaxVolume()
    local limits = {}
    limits.first = current_popup[popup_index][4]
    limits.second = current_popup[popup_index][5]
    local new = AdjustHelper("CmdEncVolumeMax", old, value, 10, limits)
    GetPreampSetting():SetMaxVolume(new)
    -- TODO readjust master volume and send slow spi
end

function CmdEncSoftLevel(index, value)
    value = EncRampHelper(10, value, 1, 2, 3, 4, 4)
    local old = GetPreampSetting():GetSoftLevel()
    local limits = {}
    limits.first = current_popup[popup_index][4]
    limits.second = current_popup[popup_index][5]
    local new = AdjustHelper("CmdEncSoftLevel", old, value, 10, limits)
    GetPreampSetting():SetSoftLevel(new)
end

function CmdEncInputBalance(index, value)
    if CheckStickyTimer() then
        return
    end
    value = EncRampHelper(GetPreampSetting():GetInputResolution(), value, 1, 2, 3, 4, 4)
    local input = GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput())
    local old = input.balance
    local new = old + (value * GetPreampSetting():GetInputResolution())
    local limits = GetPreampSetting():GetInputBalanceLimits()
    new = AdjustToLimits(new, limits)
    Log("CmdEncInputBalance: " .. old .. " " .. value .. " " .. new)
    local new = AdjustHelper("CmdEncInputBalance", old, value, 1, limits)
    if SetStickyTimer(old, new) then
        new = 0
    end
    GetPreampSetting():SetInputDevice(input.index, input.name, input.trim, new, input.polarity,
                                      input.theater_mode, input.recordable)
end

function CmdEncInputPolarity(index, value)
    Log("CmdEncInputPolarity: " .. value)
    value = EncRampHelper(10, value, 1, 1, 1, 1, 1)
    local input = GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput())
    local polarity = input.polarity
    if value < 0 then
        polarity = true
    elseif value > 0 then
        polarity = false
    end
    GetPreampSetting():SetInputDevice(input.index, input.name, input.trim, input.balance, polarity,
                                      input.theater_mode, input.recordable)
end

function CmdEncTheaterMode(index, value)
    Log("CmdEncTheaterMode: " .. value)
    value = EncRampHelper(10, value, 1, 1, 1, 1, 1)
    local input = GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput())
    if value > 0 then
        state = true
    elseif value < 0 then
        state = false
    end
    GetPreampSetting():SetInputDevice(input.index, input.name, input.trim, input.balance, input.polarity,
                                      state, input.recordable)
end

function CmdEncRecordableMode(index, value)
    Log("CmdEncRecordableMode: " .. value)
    value = EncRampHelper(10, value, 1, 1, 1, 1, 1)
    local input = GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput())
    if value > 0 then
        state = true
    elseif value < 0 then
        state = false
    end
    GetPreampSetting():SetInputDevice(input.index, input.name, input.trim, input.balance, input.polarity,
                                      input.theater_mode, state)
end

function CmdEncInputOff(index, value)
    Log("CmdEncInputOff: " .. value)
    value = EncRampHelper(10, value, 1, 1, 1, 1, 1)
    local input = GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput())
    local state = input.disabled
    if value < 0 then
        state = true
    elseif value > 0 then
        state = false
    end
    GetPreampSetting():SetInputDisabled(state)
end

function CmdEncAuxFollowsMain(index, value) 
    Log("CmdEncAuxFollowsMain: " .. value)
    value = EncRampHelper(10, value, 1, 1, 1, 1, 1)
    local state = GetPreampSetting():GetAuxFollowsMain()
    if value > 0 then
        option_commands[7] = CmdDummy
        state = true
    elseif value < 0 then
        option_commands[7] = CmdAuxSelect
        state = false
    end
    GetPreampSetting():SetAuxFollowsMain(state)
end

function CmdEncAuxPolarity(index, value)
    Log("CmdEncAuxPolarity: " .. value)
    value = EncRampHelper(10, value, 1, 1, 1, 1, 1)
    local state = GetPreampSetting():GetPolarityAux()
    if value < 0 then
        state = true
    elseif value > 0 then
        state = false
    end
    GetPreampSetting():SetPolarityAux(state)
end

function CmdEncAuxOff(index, value)
    Log("CmdEncAuxOff: " .. value)
    value = EncRampHelper(10, value, 1, 1, 1, 1, 1)
    local state = GetPreampSetting():GetAuxDisabled()
    if value < 0 then
        state = true
    elseif value > 0 then
        state = false
    end
    GetPreampSetting():SetAuxDisabled(state)
end

function CmdEncDisplayBrightness(index, value)
    value = EncRampHelper(10, value, 1, 1, 1, 1, 1)
    local old = GetPreampSetting():GetDisplayBrightness()
    local limits = {}
    limits.first = 0
    limits.second = 100
    local new = AdjustHelper("CmdEncDisplayBrightness", old, value, 5, limits)
    GetPreampSetting():SetDisplayBrightness(new)
end

function CmdEncRemoteAddress(index, value)
    value = EncRampHelper(1, value, 1, 1, 1, 1, 1)
    local old = GetPreampSetting():GetRemoteAddress()
    local limits = {}
    limits.first = 0
    limits.second = 6
    local new = AdjustHelper("CmdEncRemoteAddress", old, value, 1, limits)
    GetPreampSetting():SetRemoteAddress(new)
end

function OutputPolarityHelper(output_index, value)
    local output = GetPreampSetting():GetOutputDevice(output_index)
    value = EncRampHelper(10, value, 1, 1, 1, 1, 1)
    local state = output.polarity
    if value < 0 then
        state = true
    elseif value > 0 then
        state = false
    end
    GetPreampSetting():SetOutputDevice(output.name, output_index, output.volume, output.mute, state, output.polarity_temp)
end

function CmdEncOutput1Polarity(index, value)
    OutputPolarityHelper(0, value)
end

function CmdEncOutput2Polarity(index, value)
    OutputPolarityHelper(1, value)
end

function CmdEncOutput3Polarity(index, value)
    OutputPolarityHelper(2, value)
end

function CmdPopupHelper(number, popup_list)
    Log("POPUP HELPER")
    --if popup_list[number] ~= nil then
        if popup_index > 0 and current_popup ~= nil then
            Log("INDEX CHECK " .. popup_index .. " " .. #current_popup)
            -- transition to next popup
            popup_index = popup_index + 1
            if popup_index > #current_popup then
                ClearPopup()
                -- hacky check to remove restore default soft icon text, but not a really great place to do
                -- it otherwise
                if main_view_context.icon_region == factory_default_icons or main_view_context.icon_region == system_info_icons then
                    main_view_context.icon_region = setup_icons
                    main_view_context.icon_strs = setup_icon_strs
                    main_view_context.button_commands = setup_commands         
                end
            else
                --current_popup = popup_list[number]
                encoder_commands[4] = current_popup[popup_index][6]
                encoder_acc[4] = 0
            end
        else
            Log("STARTING POPUP")
            -- start popup
            popup_index = 1
            current_popup = popup_list[number]
            encoder_commands[4] = current_popup[popup_index][6]
            encoder_acc[4] = 0
        end
    --else
    --    Log("No popup associated with " .. number)
    --end    
end

function CmdPopup(number)
    Log("CmdPopup: " .. number)
    main_view_context.button_commands = popup_commands
    if main_view_context.icon_region == setup_icons or main_view_context.icon_region == factory_default_icons then
        CmdPopupHelper(number, popup_list_setup)
    elseif main_view_context.icon_region == option_icons then
        CmdPopupHelper(number, popup_list_option)
    elseif main_view_context.icon_region == main_icons then
        CmdPopupHelper(number, popup_list_main)
    elseif main_view_context.icon_region == system_info_icons then
        -- super hacky hardcoded check... won't be easy to add other popups to system info...
        if current_popup == popup_list_system_info[8] or number ~= 8 then
            CmdPopupHelper(number, popup_list_system_info)
        else
            -- popup helper doesn't support nested popups, so kick this one off manually
            popup_index = 1
            current_popup = popup_list_system_info[8]
            encoder_commands[4] = current_popup[popup_index][6]
            encoder_acc[4] = 0
        end
    end
    Log("New popup: " .. popup_index)
    
    return_home_time = SDL.GetTicks()
end

function ClearPopup()
    current_popup = nil
    popup_index = 0
    encoder_commands[4] = CmdEncMasterVolume
    encoder_acc[4] = 0
    if main_view_context.icon_region == main_icons then
        Log("Clearing popup to main")
        main_view_context.button_commands = main_commands
    elseif main_view_context.icon_region == option_icons then
        Log("Clearing popup to option")
        main_view_context.button_commands = option_commands
    elseif main_view_context.icon_region == setup_icons then
        Log("Clearing popup to setup")
        main_view_context.button_commands = setup_commands
    elseif main_view_context.icon_region == system_info_icons then
        Log("Clearing popup to system info")
        main_view_context.button_commands = system_info_commands     
    else
        Log("Clearing popup to NOTHING?")
    end
end

function ClearPopupAndIcons()
    -- reset soft icons and popup
    main_view_context.icon_region = main_icons
    main_view_context.icon_strs = main_icon_strs
    main_view_context.button_commands = main_commands
    ClearPopup()
end

function CmdPowerOff(number)
    Log("CmdPowerOff: " .. number)
    ClearPopupAndIcons()
    GetPreampSetting():SetStandby(true)
end

-- special command, no argument as it's just called directly
-- and is not a normal, remappable button
function CmdPowerOnSpecial()
    Log("CmdPowerOn")
    ClearPopupAndIcons()
    GetPreampSetting():SetStandby(false)
    ignore_next_button = true
end

function CmdFactoryDefaults(number)
    if number == 1 then
        Log("Restoring factory defaults")
        GetPreampSetting():RestoreFactoryDefaults()    
    end
    main_view_context.icon_region = setup_icons
    main_view_context.icon_strs = setup_icon_strs
    main_view_context.button_commands = setup_commands

    ClearPopup()
end

function CmdInstallUpdate(number)
    if number == 1 then
        Log("Installing update...")
        GetPreampSetting():InstallUpdate()
        -- TODO install the update
    end

    -- TODO need the popup to stick around....
    main_view_context.icon_region = setup_icons
    main_view_context.icon_strs = setup_icon_strs
    main_view_context.button_commands = setup_commands

    ClearPopup()
end
