--  <prjroot>/cp/display.lua

--[[

Display initialization stub for SDL draw thread

--]]

--  Required for screen positional parameters

require( "skins.position" )
require( "skins.rgbcolor" )
require( "skins.screenres" )

--  Draw loop

require( "drawdisplay" )
require("context")

--  Shared DrawRegion instances

require("draw.centeredlogo")
require("draw.softiconpanel")
require("draw.browseart")
require("draw.browselist")
require("draw.tracklist")
require("draw.source")
require("draw.trackslider")
require("draw.volume")
require("draw.nowplaying")
require("draw.screenstack")
require("draw.volumewidget")
require("draw.settingslist")
require("draw.serverlist")
require("draw.pleasewait")
require("draw.popup")
require("draw.usbconfig")
require("draw.statusbar")
require("draw.productbar")
require("draw.invalidwarning")
require("draw.networkconfirmation")
require("draw.updateconfirmation")
require("draw.changesoftwareconfirmation")

--  ContextBase instances

require("context.BROWSE")
require("context.ART_BROWSE")
require("context.PLAYLIST")
require("context.INPUT")
require("context.SETUP")
require("context.PLAY_INFO")
require("context.STANDBY")
require("context.SERVER_SELECT")
require("context.USB")
require("context.NETWORK")
require("context.UPDATE")
require("context.CHANGE_SOFTWARE")

--  Might need?

require("base.drawregion")

--require("setting")

--  BEGIN --------------------------------------------------------------

--  check for ysdl
if  not package.loaded["SDL"] or
    not SDL.ysdl_version then
        print(
            "This example code must be run using a ysdl-enabled Lua!" )
    os.exit(1)
end

screen = nil


--  SetupDisplay (used only upon initialization)

function SetupDisplay()
    -- These settings come from config.txt. Include the setting name, description, units, special unit/value 
    -- string to show when the setting is at max, min, max, default value, and optional C++ callback to be
    -- executed when the setting's value changes (defined in cpdisplaydata.cpp). Use "" to omit a callback or special max string if you don't want either.

    local data = GetDisplayData()
    local settings_ = data:GetSettings()
    local config = GetConfig()

    local s = Setting("LABEL", "Volume Settings", "LABEL", "", 0, 0, 0, "", "")
    settings_:AddSetting(s);

    s = Setting("fixed_volume", "Volume", "NO UNITS", "", 0, 1, 0, "updateVolumeMode", "Variable;Fixed")
    settings_:AddSetting(s)

    s = Setting("direct_source_selection", "Direct Source Selection", "NO UNITS", "", 0, 1, 0, "updateDirectSourceSelection", "No;Yes")
    settings_:AddSetting(s)

    s = Setting("default_volume_db", "Default Volume", "dB", "", -80, 0, -20, "", "") -- no callback since this value only gets used at startup
    settings_:AddSetting(s)

    s = Setting("max_volume_db", "Max Volume", "dB", "", -80, 0, 0, "updateMaxVolume", "")
    settings_:AddSetting(s)

    local s = Setting("LABEL", "Timer Settings", "LABEL", "", 0, 0, 0, "", "")
    settings_:AddSetting(s);

    local s = Setting("now_playing_timeout_s", "Return to Now Playing after", "seconds", "Never", 5, 60, 20, "updateNowPlayingTimeout", "")
    settings_:AddSetting(s);

    s = Setting("screen_select_timeout_s", "Hide screen select dialog after", "seconds", "", 1, 10, 10, "updateScreenSelectTimeout", "")
    settings_:AddSetting(s)

    s = Setting("standby_timeout_min", "Enter Power Saving Mode after", "minutes", "Never", 5, 125, 60, "updateStandbyTimeout", "")
    settings_:AddSetting(s)

    s = Setting("standby_mode", "Power Saving Mode", "NO UNITS", "", 0, 1, 1, "updateStandbyMode", "Low Power;High Power")
    settings_:AddSetting(s)

    local s = Setting("LABEL", "Playback Settings", "LABEL", "", 0, 0, 0, "", "")
    settings_:AddSetting(s);

    s = Setting("delay", "Delay audio by", "milliseconds", "", 20, 200, 20, "updateDelay", "")
    settings_:AddSetting(s)

    s = Setting("repeat", "Repeat", "NO UNITS", "", 0, 2, 0, "updateRepeat", "Off;Once;Continuous")
    settings_:AddSetting(s)

    local s = Setting("LABEL", "UPnP Metadata Display", "LABEL", "", 0, 0, 0, "", "")
    settings_:AddSetting(s);

    s = Setting("text_display", "Text Display", "NO UNITS", "", 0, 2, 2, "updateTextDisplay", "Scroll Always;Scroll Once;Truncate")
    settings_:AddSetting(s)

    s = Setting("show_track_num", "Show Track Number", "NO UNITS", "", 0, 1, 1, "updateShowTrackNum", "No;Yes")
    settings_:AddSetting(s)

    s = Setting("art_tracklist_toggle", "Now Playing Display", "NO UNITS", "", 0, 2, 2, "updateArtTracklistToggle", "Album Art Only;Tracklist Only;Art and Tracks")
    settings_:AddSetting(s)

    local s = Setting("LABEL", "General Settings", "LABEL", "", 0, 0, 0, "", "")
    settings_:AddSetting(s);

    s = Setting("brightness", "Brightness", "%", "", 0, 100, 100, "updateBrightness", "")
    settings_:AddSetting(s)

    s = Setting("default_browse", "Default Browse Screen", "NO UNITS", "", 0, 1, 0, "updateDefaultBrowse", "Art;List")
    settings_:AddSetting(s)

    s = Setting("remote_control_id", "Remote Control ID", "", "", 0, 6, 3, "updateRemoteControlID", "")
    settings_:AddSetting(s)

    local info = data:GetInfoString()

    if data.serial_number:find("beta") ~= nil then
        s = Setting("change_software", "Change Software Type", "CHANGE_SOFTWARE", "", 0, 0, 0, "changeSoftwareType", "")
        settings_:AddSetting(s)
        
        s = Setting("switch_partitions", "Switch Partitions", "CHANGE_SOFTWARE", "", 0, 0, 0, "switchPartitions", "")
        settings_:AddSetting(s)
    end
    
    s = Setting("LABEL", "Unit Information", "LABEL", "", 0, 0, 0, "", "")
    settings_:AddSetting(s)

    s = Setting("LABEL", info, "INFO", "", 0, 0, 0, "", "")
    settings_:AddSetting(s)

    local brightness = config:Get("brightness", "100")
    data:SetBrightness(tonumber(brightness))

    local popupYOrigin = 0 --config:Get("popup_left_y_origin", "15")
    local popupXOrigin = config:Get("popup_left_x_origin", "-150")

    ReleaseDisplayData(data)

--  Screen initialization

    screen = SDL.SetVideoMode(
        screen_width,   --  from $PRJROOT2120/cp/skins/position.lua
        screen_height,  --  ditto
        16,
        SDL.SWSURFACE)

    SDL.ShowCursor(0)

    if      not screen
        then
            Log(
                "Couldn't create screen: " .. SDL.GetError() )
            os.exit(1)
    end

    SDL.WM_SetCaption(
        "Boulder 2120 Embedded Control Point", nil )

--  Shared DrawRegion instances ----------------------------------------

    local   isVisible = true        --  set all to true
    local   animID = 1
    local   isFullscreen = true

    centeredlogo =      CenteredLogo( "centeredlogo",
                            screen_width, screen_height,
                            0, 0,
                            screen_width, screen_height,
                            0, 0,
                            isVisible,
                            animID,
                            isFullscreen )

--  Individual definitions for SoftIconPanel by context follow

    lefticons_BROWSE =      SoftIconPanel( "leftIcons_BROWSE",
                            iconSize, screen_height,
                            0, 0,
                            iconSize, screen_height,
                            0, 0,
                            isVisible,
                            AnimIDs.SCREENSELECT_ICON, AnimIDs.ARROW_UP_ICON, AnimIDs.ARROW_DOWN_ICON, AnimIDs.CHECK_ICON, AnimIDs.BACK_ICON, 
                            AnimIDs.SCREENSELECT_ICON_INV, AnimIDs.ARROW_UP_ICON_INV, AnimIDs.ARROW_DOWN_ICON_INV, AnimIDs.CHECK_ICON_INV, AnimIDs.BACK_ICON_INV)

    lefticons_ART_BROWSE =  SoftIconPanel( "leftIcons_ART_BROWSE",
                            iconSize, screen_height,
                            0, 0,
                            iconSize, screen_height,
                            0, 0,
                            isVisible,
                            AnimIDs.SCREENSELECT_ICON, AnimIDs.ART_BROWSE_LEFT, AnimIDs.ART_BROWSE_RIGHT, AnimIDs.CHECK_ICON, AnimIDs.BACK_ICON,
                            AnimIDs.SCREENSELECT_ICON_INV, AnimIDs.ART_BROWSE_LEFT_INV, AnimIDs.ART_BROWSE_RIGHT_INV, AnimIDs.CHECK_ICON_INV, AnimIDs.BACK_ICON_INV)


    lefticons_PLAYLIST =    SoftIconPanel( "lefticons_PLAYLIST",
                            iconSize, screen_height,
                            0, 0,
                            iconSize, screen_height,
                            0, 0,
                            isVisible,
                            AnimIDs.SCREENSELECT_ICON, AnimIDs.ARROW_UP_ICON, AnimIDs.ARROW_DOWN_ICON, AnimIDs.RELEASE_ICON, AnimIDs.DELETE_ICON,
                            AnimIDs.SCREENSELECT_ICON_INV, AnimIDs.ARROW_UP_ICON_INV, AnimIDs.ARROW_DOWN_ICON_INV, AnimIDs.GRAB_ICON_INV, AnimIDs.DELETE_ICON_INV)

    lefticons_SETUP =       SoftIconPanel( "lefticons_SETUP",
                            iconSize, screen_height,
                            0, 0,
                            iconSize, screen_height,
                            0, 0,
                            isVisible,
                            AnimIDs.SCREENSELECT_ICON, AnimIDs.ARROW_UP_ICON, AnimIDs.ARROW_DOWN_ICON, AnimIDs.ARROW_RIGHT_ICON, AnimIDs.ARROW_LEFT_ICON,
                            AnimIDs.SCREENSELECT_ICON_INV, AnimIDs.ARROW_UP_ICON_INV, AnimIDs.ARROW_DOWN_ICON_INV, AnimIDs.ARROW_RIGHT_ICON_INV, AnimIDs.ARROW_LEFT_ICON_INV)

    lefticons_PLAY_INFO =   SoftIconPanel( "lefticons_PLAY_INFO",
                            iconSize, screen_height,
                            0, 0,
                            iconSize, screen_height,
                            0, 0,
                            isVisible,
                            AnimIDs.SCREENSELECT_ICON, AnimIDs.PREVIOUS_ICON, AnimIDs.NEXT_ICON, AnimIDs.PLAY_ICON, AnimIDs.EDIT_PLAYLIST,
                            AnimIDs.SCREENSELECT_ICON_INV, AnimIDs.PREVIOUS_ICON_INV, AnimIDs.NEXT_ICON_INV, AnimIDs.PLAY_ICON_INV, AnimIDs.EDIT_PLAYLIST_INV)

    lefticons_SERVER_SELECT =   SoftIconPanel( "lefticons_PLAY_INFO",
                            iconSize, screen_height,
                            0, 0,
                            iconSize, screen_height,
                            0, 0,
                            isVisible,
                            AnimIDs.SCREENSELECT_ICON, AnimIDs.ARROW_UP_ICON, AnimIDs.ARROW_DOWN_ICON, AnimIDs.CHECK_ICON, AnimIDs.BLANK_LEFT_5,
                            AnimIDs.SCREENSELECT_ICON_INV, AnimIDs.ARROW_UP_ICON_INV, AnimIDs.ARROW_DOWN_ICON_INV, AnimIDs.CHECK_ICON_INV, AnimIDs.BLANK_LEFT_5)

    lefticons_USB =         SoftIconPanel( "lefticons_PLAY_INFO",
                            iconSize, screen_height,
                            0, 0,
                            iconSize, screen_height,
                            0, 0,
                            isVisible,
                            AnimIDs.BLANK_LEFT_1, AnimIDs.ARROW_UP_ICON, AnimIDs.ARROW_DOWN_ICON, AnimIDs.ARROW_RIGHT_ICON, AnimIDs.BLANK_LEFT_5,
                            AnimIDs.BLANK_LEFT_1, AnimIDs.ARROW_UP_ICON_INV, AnimIDs.ARROW_DOWN_ICON_INV, AnimIDs.ARROW_RIGHT_ICON_INV, AnimIDs.BLANK_LEFT_5)

    righticons_USB =        SoftIconPanel( "righticons_USB",
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            isVisible,
                            AnimIDs.VOLUME_UP_ICON, AnimIDs.VOLUME_DOWN_ICON, AnimIDs.MUTE_ICON, AnimIDs.R4_CHECK_INACTIVE, AnimIDs.STANDBY_ICON,
                            AnimIDs.VOLUME_UP_ICON_INV, AnimIDs.VOLUME_DOWN_ICON_INV, AnimIDs.MUTE_ICON_INV, AnimIDs.R4_CHECK_ACTION, AnimIDs.STANDBY_ICON_INV)

    righticons_PLAY_INFO =  SoftIconPanel( "righticons_PLAY_INFO",
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            isVisible,
                            AnimIDs.VOLUME_UP_ICON, AnimIDs.VOLUME_DOWN_ICON, AnimIDs.MUTE_ICON, AnimIDs.CLEAR_PLAYLIST_ICON, AnimIDs.STANDBY_ICON,
                            AnimIDs.VOLUME_UP_ICON_INV, AnimIDs.VOLUME_DOWN_ICON_INV, AnimIDs.MUTE_ICON_INV, AnimIDs.CLEAR_PLAYLIST_ICON_INV, AnimIDs.STANDBY_ICON_INV)

    righticons_SERVER_SELECT =  SoftIconPanel( "righticons_SERVER_SELECT",
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            isVisible,
                            AnimIDs.VOLUME_UP_ICON, AnimIDs.VOLUME_DOWN_ICON, AnimIDs.MUTE_ICON, AnimIDs.BLANK_RIGHT, AnimIDs.STANDBY_ICON,
                            AnimIDs.VOLUME_UP_ICON_INV, AnimIDs.VOLUME_DOWN_ICON_INV, AnimIDs.MUTE_ICON_INV, AnimIDs.BLANK_RIGHT, AnimIDs.STANDBY_ICON_INV)

    righticons_PLAYLIST =  SoftIconPanel( "righticons_PLAYLIST",
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            isVisible,
                            AnimIDs.VOLUME_UP_ICON, AnimIDs.VOLUME_DOWN_ICON, AnimIDs.MUTE_ICON, AnimIDs.BLANK_RIGHT, AnimIDs.STANDBY_ICON,
                            AnimIDs.VOLUME_UP_ICON_INV, AnimIDs.VOLUME_DOWN_ICON_INV, AnimIDs.MUTE_ICON_INV, AnimIDs.BLANK_RIGHT, AnimIDs.STANDBY_ICON_INV)

    righticons_BROWSE =  SoftIconPanel( "righticons_BROWSE",
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            isVisible,
                            AnimIDs.VOLUME_UP_ICON, AnimIDs.VOLUME_DOWN_ICON, AnimIDs.MUTE_ICON, AnimIDs.CHILD_ADD, AnimIDs.STANDBY_ICON,
                            AnimIDs.VOLUME_UP_ICON_INV, AnimIDs.VOLUME_DOWN_ICON_INV, AnimIDs.MUTE_ICON_INV, AnimIDs.CHILD_ADD_INV, AnimIDs.STANDBY_ICON_INV)

    righticons_SETUP =  SoftIconPanel( "righticons_SETUP",
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            iconSize, screen_height,
                            screen_width - iconSize, 0,
                            isVisible,
                            AnimIDs.VOLUME_UP_ICON, AnimIDs.VOLUME_DOWN_ICON, AnimIDs.MUTE_ICON, AnimIDs.RESET_DEFAULTS_INACTIVE, AnimIDs.STANDBY_ICON,
                            AnimIDs.VOLUME_UP_ICON_INV, AnimIDs.VOLUME_DOWN_ICON_INV, AnimIDs.MUTE_ICON_INV, AnimIDs.RESET_DEFAULTS_ACTION, AnimIDs.STANDBY_ICON_INV)

    nowplaying =        NowPlaying( "nowplaying",
                            screen_width, screen_height,
                            0, 0,
                            iconSize, screen_height,
                            0, 0,
                            isVisible)

    nowplaying:InitPosition()

    browseart =         BrowseArt( "browseart",
                            0, 0,
                            0, 0,
                            0, 0,
                            0, 0,   --  class init
                            isVisible)

    browseart:InitPosition()

    browselistRegion =  BrowseList( "browselist",
                            0, 0,
                            0, 0,
                            0, 0,
                            0, 0,   --  class init
                            isVisible)

    tracklistRegion2 = Tracklist( "tracklist2",
                            0, 0,
                            0, 0,
                            0, 0,
                            0, 0,   --  class init
                            isVisible,
                            true,
                            false)

    local   initSourceIndex = 0
    local   initSourceName = "Network"
    local   initSourceType = "Playlist"

    sourceRegion =      Source( "SourceDisplay",
                            sourceWidth*3, sourceHeight,
                            iconSize, screen_height - sourceHeight,
                            sourceWidth*3, sourceHeight,
                            iconSize, screen_height - sourceHeight,
                            false,

                            initSourceIndex,
                            initSourceName,
                            initSourceType )

    statusBar =         StatusBar("statusBar",
                            screen_width - (2*iconSize), 30,
                            iconSize, screen_height - 30,
                            screen_width - (2*iconSize), 30,
                            iconSize, screen_height - 30,
                            true)

    productBar =        ProductBar("productBar",
                            screen_width - (2*iconSize), 30,
                            iconSize, 0,
                            screen_width - (2*iconSize), 30,
                            iconSize, 0,
                            true)

    trackSliderNowPlaying = TrackSlider("trackSliderNowPlaying",
                            700, 25,       -- frame width, height
                            500, 410, -- frame xpos, ypos
                            700, 25,       -- bound width, height
                            500, 410, -- bound xpos, ypos
                            true)                    -- is visible

    tracklistNowPlaying   = Tracklist("tracklistNowPlaying",
                            700, 320, -- frame width, height
                            500, 60,  -- frame xpos, ypos
                            700, 320, -- bound width, height
                            500, 60,  -- bound xpos, ypos
                            true)     -- is visible

    screenStack = ScreenStack("screenStack",
                            300, 480,            -- frame width, height
                            iconSize, 0, -- frame xpos, ypos
                            300, 480,            -- bound width, height
                            iconSize, 0, -- bound xpos, ypos
                            true)

    volumeWidget = VolumeWidget("volumeWidget",
                            300, 480, -- frame width, height
                            900, 0,  -- frame xpos, ypos
                            300, 480, -- bound width, height
                            900, 0,  -- bound xpos, ypos
                            true)

    settingsList = SettingsList("settingsList",
                            screen_width - iconSize*2, screen_height, -- frame width, height
                            iconSize, 0, -- frame xpos, ypos
                            screen_width - iconSize*2, screen_height, -- bound width, height
                            iconSize, 0, -- bound xpos, ypos
                            true)

    serverList = ServerList("serverList",
                            screen_width - iconSize*2, screen_height, -- frame width, height
                            iconSize, 0, -- frame xpos, ypos
                            screen_width - iconSize*2, screen_height, -- bound width, height
                            iconSize, 0, -- bound xpos, ypos
                            true)

    pleaseWait = PleaseWait("pleaseWait",
                            240, 240, -- frame width, height
                            (screen_width/2) - (240/2), (screen_height/2) - (240/2),  -- frame xpos, ypos
                            240, 240, -- bound width, height
                            (screen_width/2) - (240/2), (screen_height/2) - (240/2),  -- bound xpos, ypos
                            true)

    popUp =           PopUp("popUp",
                            300, 480, -- frame width, height
                            popupXOrigin, popupYOrigin,  -- frame xpos, ypos
                            300, 480, -- bound width, height
                            popupXOrigin, popupYOrigin,  -- bound xpos, ypos
                            true)

    invalidWarning = InvalidWarning("invalidWarning",
                            screen_width - (2*iconSize), 30, -- frame width, height
                            iconSize, (screen_height) - 30,  -- frame xpos, ypos
                            screen_width - (2*iconSize), 30, -- bound width, height
                            iconSize, (screen_height) - 30,  -- bound xpos, ypos
                            true)

    usbConfig =    UsbConfig("usbConfig",
                            screen_width - iconSize*2, screen_height, -- frame width, height
                            iconSize, 0, -- frame xpos, ypos
                            screen_width - iconSize*2, screen_height, -- bound width, height
                            iconSize, 0, -- bound xpos, ypos
                            true)

    networkConfirmation =   NetworkConfirmation("networkConfirmation",
                            screen_width - iconSize*2, screen_height, -- frame width, height
                            iconSize, 0, -- frame xpos, ypos
                            screen_width - iconSize*2, screen_height, -- bound width, height
                            iconSize, 0, -- bound xpos, ypos
                            true)

    updateConfirmation =    UpdateConfirmation("updateConfirmation",
                            screen_width - iconSize*2, screen_height, -- frame width, height
                            iconSize, 0, -- frame xpos, ypos
                            screen_width - iconSize*2, screen_height, -- bound width, height
                            iconSize, 0, -- bound xpos, ypos
                            true)

    changeSoftwareConfirmation = ChangeSoftwareConfirmation("changeSoftwareConfirmation",
                                 screen_width - iconSize*2, screen_height, -- frame width, height
                                 iconSize, 0, -- frame xpos, ypos
                                 screen_width - iconSize*2, screen_height, -- bound width, height
                                 iconSize, 0, -- bound xpos, ypos
                                 true)

--  Region context initializations follow ------------------------------

    BrowseList =        BROWSE(     nil,
                                    lefticons_BROWSE,
                                    righticons_BROWSE,
                                    browselistRegion,
                                    tracklistRegion2)

    ArtBrowse =         ART_BROWSE( nil,
                                    lefticons_ART_BROWSE,
                                    righticons_BROWSE,
                                    browseart )

    Setup =             SETUP(      nil,
                                    lefticons_SETUP,
                                    righticons_SETUP,
                                    settingsList)

    NowPlaying =        PLAY_INFO(  nil,
                                    lefticons_PLAY_INFO,
                                    righticons_PLAY_INFO,
                                    nowplaying,
                                    trackSliderNowPlaying,
                                    tracklistNowPlaying)

    Standby =           STANDBY(    nil )

    ServerSelect =      SERVER_SELECT(nil,
                                    lefticons_SERVER_SELECT,
                                    righticons_SERVER_SELECT,
                                    serverList)

    Usb =               USB(        nil,
                                    lefticons_USB,
                                    righticons_USB,
                                    usbConfig)

    Network =           NETWORK(    nil,
                                    lefticons_USB,
                                    righticons_USB,
                                    networkConfirmation)

    Update =            UPDATE(    nil,
                                    lefticons_USB,
                                    righticons_USB,
                                    updateConfirmation)

    ChangeSoftware = CHANGE_SOFTWARE(nil,
                                     lefticons_USB,
                                     righticons_USB,
                                     changeSoftwareConfirmation)

    testDrawn = false

    return screen
end
