--  <prjroot>/cp/draw/softiconpanel.lua

--[[    Soft Icon Panel        DrawRegion class

Renders flexible-purpose icons to left side of display.

--]]

require("base.drawregion")
require("base.iconregion")
require("base.rectangle")
require("draw.singlelinetext")

SoftTextPanel = class( DrawRegion,
                        function( 
                            a, name,
                            frameWidth, frameHeight, 
                            frameLeftXpos, frameTopYpos,
                            isVisible, numIcons,
                            iconStrs -- a list of strings to use for initial descriptions
                        )

                            DrawRegion.init(
                                a, name,
                                frameWidth, frameHeight, 
                                frameLeftXpos, frameTopYpos,
                                isVisible )

                            a.iconStrs = iconStrs
                            print("top " .. frameTopYpos)

                            a.inverted = {}
                            for i = 1, numIcons do
                                a.inverted = false
                            end
                        end )


function SoftTextPanel:Draw( displayData, upnpData, noclear )
    if noclear == nil then
        noclear = false
    end

    if noclear == false then
        Display.FillBackground(self.frameLeftXpos, self.frameTopYpos, self.frameWidth, self.frameHeight)
    end
    self:DrawAllIcons()
end

function SoftTextPanel:OnExit()
    Display.FillBackground(self.frameLeftXpos, self.frameTopYpos, self.frameWidth, self.frameHeight)
end


function SoftTextPanel:GetIconCoordsByPosition( position )

    local   iconCoords = { x = 0, y = 0 }
    local   paddingBetweenIcons = 0

    local   index = math.floor( position ) - 1

    local   offsetYcoords = self.frameTopYpos
    local   offsetXcoords = index * ( paddingBetweenIcons + iconSizeWidth) + 10

    if index > 5 then
        offsetXcoords = offsetXcoords + 37
    end

    if index > 8 then
        offsetXcoords = offsetXcoords + 37
    end

    -- Set values for return
    iconCoords.x = offsetXcoords
    iconCoords.y = offsetYcoords

    return  iconCoords
        
end

function SoftTextPanel:DrawAllIcons()

    for k, v in pairs(self.iconStrs) do
        self:DrawIndividualText(k, v)
    end

    if main_view_context.icon_region == main_icons or
        main_view_context.icon_region == option_icons or
        main_view_context.icon_region == aux_select_icons then
            if not GetPreampSetting():GetAuxDisabled() then
                local coords = self:GetIconCoordsByPosition(GetPreampSetting():GetCurrentAux() + 1)
                Display.DrawRectangle(coords.x, coords.y + 1, iconSizeWidth, iconSizeHeight - 1, MINT)
                Display.DrawRectangle(coords.x + 1, coords.y + 2, iconSizeWidth, iconSizeHeight - 3, MINT)
            end
    end

    if main_view_context.icon_region == option_icons and GetPreampSetting():GetAuxFollowsMain() then
        local coords = self:GetIconCoordsByPosition(7)
        Display.DrawLine(coords.x + 5, coords.y + 5, coords.x + iconSizeWidth - 5, coords.y + iconSizeHeight - 5, CREME)
        Display.DrawLine(coords.x + 5, coords.y + iconSizeHeight - 5, coords.x + iconSizeWidth - 5, coords.y + 5, CREME)
    end
end

function SoftTextPanel:DrawIndividualText( position, text)
    local coords = self:GetIconCoordsByPosition(position)

    local input_device = nil
    if position < 7 then
        input_device = GetPreampSetting():GetInputDevice(position - 1)
        if text == "INPUT" then
            text = input_device.name
        end
    end
    local is_highlighted = false

    if position == 8 then
        is_highlighted = GetPreampSetting():GetMute()
    end
    if main_view_context.icon_region ~= setup_icons and main_view_context.icon_region ~= factory_default_icons and main_view_context.icon_region ~= system_info_icons and main_view_context.icon_region ~= aux_select_icons then
        if input_device ~= nil then
            --print("current: " .. GetPreampSetting():GetCurrentInput() .. " index: " .. input_device.index)
            if position - 1 == GetPreampSetting():GetCurrentInput() and not GetPreampSetting():GetInputDisabled() then
                is_highlighted = true
            end
        end
    end
    if main_view_context.icon_region == option_icons then
        if position == 10 then
            is_highlighted = GetPreampSetting():GetOutputDevice(0).polarity_temp
        end
        if position == 11 then
            is_highlighted = GetPreampSetting():GetOutputDevice(1).polarity_temp
        end
        if position == 12 then
            is_highlighted = GetPreampSetting():GetOutputDevice(2).polarity_temp
        end
        if position == 14 then
            is_highlighted = GetPreampSetting():GetPolarity()
        end
        if position == 15 then
            is_highlighted = GetPreampSetting():GetPolarityRight()
        end
    end
    if main_view_context.icon_region == main_icons then
        if position == 7 then
            is_highlighted = GetPreampSetting():GetSoft()
        end
        if position == 10 then
            is_highlighted = GetPreampSetting():GetOutputDevice(0).mute
        end
        if position == 11 then
            is_highlighted = GetPreampSetting():GetOutputDevice(1).mute
        end
        if position == 12 then
            is_highlighted = GetPreampSetting():GetOutputDevice(2).mute
        end
    end
    if main_view_context.icon_region == setup_icons then
        if position == 4 then
            is_highlighted = GetPreampSetting():GetInputDisabled()
        end
        if position == 6 then
            is_highlighted = GetPreampSetting():GetAuxDisabled()
        end
    end
    if encoder_commands[4] == CmdEncVolumeScale then
        if main_view_context.icon_region == setup_icons then
            if position == 10 then
                is_highlighted = true
            end
        end
    end
    
    local font_id = 1
    if is_highlighted then
        --font_id = 2
        Display.DrawRectangleFilled(coords.x, coords.y, iconSizeWidth, iconSizeHeight, HIGHLIGHT)
    else
        Display.DrawRectangleFilled(coords.x, coords.y, iconSizeWidth, iconSizeHeight, ROAST)
    end

    local words = {}
    for word in text:gmatch("%g+") do
        if word ~= nil then
            table.insert(words, word)
        end
    end
    -- special case for power button
    if text == "Power" then
        local power_icon = Display.AcquireImage(2)
        local power_rect = SDL.newRect(coords.x + 6, coords.y - 5, iconSizeWidth, iconSizeHeight)
        SDL.BlitSurface(power_icon, nil, screen, power_rect)
    else
        -- normal case, up to three lines of text
        if #words == 0 then
            -- do nothing
        elseif #words == 1 then
            SingleLineText(words[1], font_id, coords.x + 2, coords.y + 4, iconSizeWidth - 4, iconSizeHeight - 4, 0, "center", 0, false, false)
        elseif #words == 2 then
            SingleLineText(words[1], font_id, coords.x + 2, coords.y + 4 + (1 * (iconSizeHeight - 4) / 3) - (iconSizeHeight - 4) / 6, iconSizeWidth - 4, (iconSizeHeight - 4) / 3, 0, "center", 0, false, false)
            SingleLineText(words[2], font_id, coords.x + 2, coords.y + 4 + (2 * (iconSizeHeight - 4) / 3) - (iconSizeHeight - 4) / 6, iconSizeWidth - 4, (iconSizeHeight - 4) / 3, 0, "center", 0, false, false)
        else
            SingleLineText(words[1], font_id, coords.x + 2, coords.y + 4 + (0 * ((iconSizeHeight - 4) / 3)), iconSizeWidth - 4, (iconSizeHeight - 4) / 3, 0, "center", 0, false, false)
            SingleLineText(words[2], font_id, coords.x + 2, coords.y + 4 + (1 * ((iconSizeHeight - 4) / 3)), iconSizeWidth - 4, (iconSizeHeight - 4) / 3, 0, "center", 0, false, false)
            SingleLineText(words[3], font_id, coords.x + 2, coords.y + 4 + (2 * ((iconSizeHeight - 4) / 3)), iconSizeWidth - 4, (iconSizeHeight - 4) / 3, 0, "center", 0, false, false)
            
        end
    end

    -- borders around boxes
    Display.DrawRectangle(coords.x, coords.y + 1, iconSizeWidth, iconSizeHeight - 1, CREME)
end

function SoftTextPanel:DrawIndividualIcon( position, animID )

    local   coords = self:GetIconCoordsByPosition( position )

    local   icon = IconRegion( ("Icon " .. position),
                    iconSizeWidth, iconSizeHeight,
                    coords.x, coords.y,
                    true, animID )
    icon:Draw()
end

function SoftTextPanel:WipeIndividualIconBackground( position )

    local   coords = self:GetIconCoordsByPosition( position )

    local   wipeColor = grey5
    local   wipeFilled = true
    local   isVisible = true

    local   iconBkgdWipe = Rectangle(   "iconBkgdWipe " .. position,
                                        iconSizeWidth - 1, iconSizeHeight,
                                        coords.x, coords.y,
                                        wipeColor, wipeFilled,
                                        isVisible )

    iconBkgdWipe:Draw() 
end
