
function DisplayBrightness()
    return GetPreampSetting():GetDisplayBrightness()
end

function NumberString(val)
    return string.format("%d", val)
end

-- list is indexed by button number
popup_list_main = {}
popup_list_main[9] = { { "Display Brightness", NumberString, DisplayBrightness, 0, 100, CmdEncDisplayBrightness } }

popup_list_option = {}
popup_list_option[9] = { { "Display Brightness", NumberString, DisplayBrightness, 0, 100, CmdEncDisplayBrightness } }

function VolumeRes()
    return GetPreampSetting():GetVolumeResolution()
end

function VolumeScale()
    return GetPreampSetting():GetVolumeScale()
end

function VolumeScaleString(val)
    return string.format("%d to %d", -100 + val, 0 + val)
end

function VolumeDefault()
    return GetPreampSetting():GetDefaultVolume()
end

function VolumeDefaultString(val)
    if val == -490 then
        return "Off"
    else
        return VolumeString(val)
    end
end

function VolumeMax()
    return GetPreampSetting():GetMaxVolume()
end

function VolumeResolutionString(res)
    local str = ""
    if res == VolumeResolution.RES_0_1 then
        str = "0.1 dB"
    elseif res == VolumeResolution.RES_0_5 then
        str = "0.5 dB"
    else
        str = "1.0 dB"
    end
    return str
end

function InputBalance()
    local bal = GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput()).balance
    return bal
end

function InputPolarity()
    if not GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput()).polarity then
        return 0
    else
        return 1
    end
end

function PolarityString(val)
    if val == 0 or val == false then
        return "Pin 2 Hot"
    else
        return "Pin 3 Hot"
    end
end

function OnOffString(val)
    if val == true then
        return "On"
    else
        return "Off"
    end
end

function SoftLevel()
    return GetPreampSetting():GetSoftLevel()
end

function InputOn()
    return not GetPreampSetting():GetInputDisabled()
end

function InputRecordableMode()
    local input = GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput())
    return input.recordable
end

function InputTheaterMode()
    local input = GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput())
    return input.theater_mode
end

function AuxFollowsMain()
    return GetPreampSetting():GetAuxFollowsMain()
end

function AuxPolarity()
    return GetPreampSetting():GetPolarityAux()
end

function Output1Polarity()
    local output = GetPreampSetting():GetOutputDevice(0)
    return output.polarity
end

function Output2Polarity()
    local output = GetPreampSetting():GetOutputDevice(1)
    return output.polarity
end

function Output3Polarity()
    local output = GetPreampSetting():GetOutputDevice(2)
    return output.polarity
end

function AuxOn()
    return not GetPreampSetting():GetAuxDisabled()
end

function RemoteAddress()
    return GetPreampSetting():GetRemoteAddress()
end

function FactoryDefaults()
    if main_view_context.icon_region ~= factory_default_icons then
        main_view_context.icon_region = factory_default_icons
        main_view_context.icon_strs = factory_default_icon_strs
        main_view_context.button_commands = factory_default_commands
    end
end

function SysInfo()
    if main_view_context.icon_region ~= system_info_icons then
        main_view_context.icon_region = system_info_icons
        main_view_context.icon_strs = system_info_icon_strs
        main_view_context.button_commands = system_info_commands
    end    
end

popup_list_setup = {}
popup_list_setup[2] = { { "Input Balance", InputBalanceString, InputBalance, GetPreampSetting():GetBalanceLimits().first, GetPreampSetting():GetBalanceLimits().second, CmdEncInputBalance },
                        { "Input Polarity", PolarityString, InputPolarity, 0, 1, CmdEncInputPolarity } }
popup_list_setup[3] = { { "Input Theater Mode", OnOffString, InputTheaterMode, 0, 1, CmdEncTheaterMode },
                        { "Input Recordable Mode", OnOffString, InputRecordableMode, 0, 1, CmdEncRecordableMode } }
popup_list_setup[4] = { { "Input", OnOffString, InputOn, 0, 1, CmdEncInputOff } }
popup_list_setup[5] = { { "Aux Follows Main", OnOffString, AuxFollowsMain, 0, 1, CmdEncAuxFollowsMain },
                        { "Aux Polarity", PolarityString, AuxPolarity, 0, 1, CmdEncAuxPolarity } }
popup_list_setup[6] = { { "Aux Output", OnOffString, AuxOn, 0, 1, CmdEncAuxOff } }
popup_list_setup[7] = { { "Soft Level", VolumeString, SoftLevel, GetPreampSetting():GetSoftLimits().first, GetPreampSetting():GetSoftLimits().second, CmdEncSoftLevel } }
popup_list_setup[9] = { { "Output 1 Polarity", PolarityString, Output1Polarity, 0, 1, CmdEncOutput1Polarity },
                        { "Output 2 Polarity", PolarityString, Output2Polarity, 0, 1, CmdEncOutput2Polarity },
                        { "Output 3 Polarity", PolarityString, Output3Polarity, 0, 1, CmdEncOutput3Polarity } }
popup_list_setup[10] = { { "Volume Scale", VolumeScaleString, VolumeScale, 0, 100, CmdEncVolumeScale },
                         { "Volume Resolution", VolumeResolutionString, VolumeRes, 0, 2, CmdEncVolumeRes } }
popup_list_setup[12] = { { "Volume Default", VolumeDefaultString, VolumeDefault, -900, -490, CmdEncVolumeDefault},
                         { "Volume Max", VolumeString, VolumeMax, -200, 0, CmdEncVolumeMax } }
-- these are not 'real' popups, they have special-cased drawing and no encoder function
popup_list_setup[13] = { { "Sys Info", NumberString, SysInfo, 0, 0, CmdDummy } }
popup_list_setup[15] = { { "Restore Factory Defaults", NumberString, FactoryDefaults, 0, 0, CmdDummy } } 

popup_list_system_info = {}
-- TODO: proper remote address strings and commands
popup_list_system_info[8] = { { "Remote Address", NumberString, RemoteAddress, 0, 6, CmdEncRemoteAddress } }

popup_index = 0
