
package.path = package.path .. ';lua/?.lua'

if not package.loaded["SDL"] or not SDL.ysdl_version then
    print("This example code must be run using a ysdl-enabled Lua!" )
    os.exit(1)
end

screen = nil
rotate = nil
bg = nil
screen_width = 1280
screen_height = 240
buttons = nil

require("button_init")
require("context_init")
require("popup_init")
require("draw.error_view")

function SetupDisplay()
    rotate = SDL.SetVideoMode(screen_width, screen_height, 16, SDL.SWSURFACE)
    SDL.ShowCursor(0)

    if not rotate then
        Log("Couldn't create screen: " .. SDL.GetError())
        os.exit(1)
    end

    SDL.WM_SetCaption("Boulder 3010 Preamp", nil)

    bg = SDL.CreateRGBSurface(SDL.SWSURFACE, screen_width, screen_height, 32, 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000)
    bg = SDL.DisplayFormat(bg)

    screen = SDL.CreateRGBSurface(SDL.SWSURFACE, screen_width, screen_height, 32, 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000)
    screen = SDL.DisplayFormat(screen)
    
    Display.SetDisplayPointers(screen, bg)

end

function PreampSetup()

    -- load background
    bg_img = Display.AcquireImage(4)
    SDL.BlitSurface(bg_img, nil, bg, nil)

    standby_img = Display.AcquireImage(9)

    -- register keys to buttons
    buttons = {}
    buttons[SDL.K_1] = 1
    buttons[SDL.K_2] = 2
    buttons[SDL.K_3] = 3
    buttons[SDL.K_4] = 4
    buttons[SDL.K_5] = 5
    buttons[SDL.K_6] = 6
    buttons[SDL.K_7] = 7
    buttons[SDL.K_8] = 8
    buttons[SDL.K_9] = 9
    buttons[SDL.K_0] = 10
    buttons[SDL.K_q] = 11
    buttons[SDL.K_w] = 12
    buttons[SDL.K_e] = 13
    buttons[SDL.K_r] = 14
    buttons[SDL.K_t] = 15

    -- directional encoding (positive/negative) of the encoder index for pc simulation
    encoder_buttons = {}
    encoder_buttons[SDL.K_a] = 1
    encoder_buttons[SDL.K_z] = -1
    encoder_buttons[SDL.K_s] = 2
    encoder_buttons[SDL.K_x] = -2
    encoder_buttons[SDL.K_d] = 3
    encoder_buttons[SDL.K_c] = -3
    encoder_buttons[SDL.K_f] = 4
    encoder_buttons[SDL.K_v] = -4
    encoder_buttons[SDL.K_g] = 5
    encoder_buttons[SDL.K_b] = -5
    encoder_buttons[SDL.K_h] = 6
    encoder_buttons[SDL.K_n] = -6
    encoder_buttons[SDL.K_j] = 7
    encoder_buttons[SDL.K_m] = -7

    -- register encoder commands for main view
    encoder_commands = {}
    encoder_commands[1] = CmdEncInputTrim
    encoder_commands[2] = CmdEncBlend
    encoder_commands[3] = CmdEncBalance
    encoder_commands[4] = CmdEncMasterVolume
    encoder_commands[5] = CmdEncOutputVolume
    encoder_commands[6] = CmdEncOutputVolume
    encoder_commands[7] = CmdEncOutputVolume
    encoder_acc = {}
    encoder_acc[1] = 0
    encoder_acc[2] = 0
    encoder_acc[3] = 0
    encoder_acc[4] = 0
    encoder_acc[5] = 0
    encoder_acc[6] = 0
    encoder_acc[7] = 0
    
    -- table to track held keys for pc encoder simulation
    held_encoders = {}

    -- remote commands are fixed function (yay!)
    remote_commands = {}
    remote_commands[0x4] = CmdRemoteStandby
    remote_commands[0x7] = CmdRemoteBrightness
    remote_commands[0x10] = CmdRemoteVolumeDown
    remote_commands[0x13] = CmdRemoteVolumeUp
    remote_commands[0x14] = CmdRemoteBalanceRight
    remote_commands[0x15] = CmdRemoteBalanceLeft
    remote_commands[0x12] = CmdRemoteMute
    remote_commands[0x1f] = CmdRemoteSoftMute
    remote_commands[0x2f] = CmdRemotePolarity
    remote_commands[0x37] = CmdRemoteRightPolarity
    remote_commands[0x31] = CmdRemoteBlendDown
    remote_commands[0x29] = CmdRemoteBlendUp
    remote_commands[0x36] = CmdRemoteInputTrimDown
    remote_commands[0x2e] = CmdRemoteInputTrimUp
    remote_commands[0x3c] = CmdRemoteOutput1Down
    remote_commands[0x34] = CmdRemoteOutput1Up
    remote_commands[0x3b] = CmdRemoteOutput2Down
    remote_commands[0x33] = CmdRemoteOutput2Up
    remote_commands[0x3a] = CmdRemoteOutput3Down
    remote_commands[0x32] = CmdRemoteOutput3Up
    remote_commands[0x2d] = CmdRemoteOutput1Mute
    remote_commands[0x35] = CmdRemoteOutput2Mute
    remote_commands[0x3d] = CmdRemoteOutput3Mute
    remote_commands[0x27] = CmdRemoteInput1
    remote_commands[0x21] = CmdRemoteInput2
    remote_commands[0x22] = CmdRemoteInput3
    remote_commands[0x20] = CmdRemoteInput4
    remote_commands[0x23] = CmdRemoteInput5
    remote_commands[0x24] = CmdRemoteInput6
    
    current_popup = nil
    last_config_time = nil
    command_sticky_time = nil
    return_home_time = nil
    last_standby_time = SDL.GetTicks()
    ignore_next_button = false
    last_mute_time = SDL.GetTicks()
    disable_encoder_acc = false -- acc is annoying for pc sim

    if GetPreampSetting():GetAuxFollowsMain() then
        option_commands[7] = CmdDummy
    end

    update = GetPreampSetting():CheckForUpdates()
    GetPreampSetting():SetStandby(false)
    GetPruInterface():CalculateSlowSpi()
end

function PreampMain()
    local start_ticks = SDL.GetTicks()
    
    local config = GetConfigLock()
    local preamp = GetPreampSettingLock()
    local dm = GetDisplayManagerLock()

    if GetPreampSetting():GetStandby() then
        -- standby special case
        --Display.FillBackground(0, 0, screen_width, screen_height)
        --SingleLineText("I'm in standby!", 52, 50, 50, 1180, 100, 0, "center", 0, false, false)
        SDL.BlitSurface(standby_img, nil, screen, nil)
        last_standby_time = SDL.GetTicks()
    else
        -- main screen drawing
        local r = SDL.newRect(0, 0, screen_width, screen_height)
        main_view_context:Draw()
        if current_popup ~= nil then
            popup_view_context:Draw()
        end
        --if GetPreampSetting():GetMostRecentError() ~= "" then
        --    Log("Most recent error was not empty")
        --    error_view_region:Draw() 
        --end
        update = GetPreampSetting().update_ready
        if GetPreampSetting():GetUpdateInProgress() then
            local percent = GetPreampSetting():GetUpdatePercent()
            local status = GetPreampSetting():GetUpdateStatus()
            SingleLineText(status..percent.."%", 6, 800, 4, 450, 20, 0, "center", 0, false, false)
        elseif update then
            --Log("update available")
            SingleLineText("Update Available", 6, 960, 4, 200, 20, 0, "center", 0, false, false)
        end    
        
    end

    Display.Rotate(screen, rotate)
    SDL.UpdateRect(rotate, 0, 0, 0, 0)

    -- keyboard handling
    local event = SDL.PollEvent()
    while event ~= nil do
        if event.type == SDL.KEYDOWN then
            
            if return_home_time ~= nil then
                return_home_time = SDL.GetTicks()
            end
            --Log("new key event: " .. event.sym .. " " .. SDL.K_p)
            if event.sym == SDL.K_ESCAPE then
                Log("Escape pressed, exiting")
                EmergencyShutdown()
            elseif event.sym == SDL.K_p and GetPreampSetting():GetStandby() then -- special case -- buttons don't really work in standby mode, we will just get the interrupt... and we may get more than one keypress
                                             -- so there is a special key for this case
                Log("Special power key!")
                CmdPowerOnSpecial()
            else
                if event.sym ~= nil and
                event.sym ~= SDL.K_t then -- special case power off to happen on release
                    if buttons[event.sym] ~= nil then
                        if ignore_next_button == false then
                            main_view_context.button_commands[buttons[event.sym]](buttons[event.sym])
                        else
                            ignore_next_button = false
                        end
                    end
                    if encoder_buttons[event.sym] ~= nil then
                        held_encoders[event.sym] = true
                    end
                end
            end
        elseif event.type == SDL.KEYUP then
            if event.sym ~= nil then
                -- power off button happens on release, because any buttons press will take it
                -- out of standby, and we get that event even with a quick power off press
                if event.sym == SDL.K_t then
                    if ignore_next_button == false then
                        main_view_context.button_commands[buttons[event.sym]](buttons[event.sym])
                    else
                        ignore_next_button = false
                    end
                else
                    held_encoders[event.sym] = false
                end
            end
        end
        event = SDL.PollEvent()
    end

    -- pc encoder emulation via keyboard
    for k, v in pairs(held_encoders) do
        if v == true then
            local encoder = math.abs(encoder_buttons[k])
            local old_value = GetPruInterface():GetEncoderValue(encoder - 1)
            local mod_value = 0
            if encoder_buttons[k] > 0 then
                mod_value = 1
            else
                mod_value = -1
            end
            
            GetPruInterface():SetEncoderValue(encoder - 1, mod_value)
        end

        -- disable acc for pc
        disable_encoder_acc = true
    end

    --disable_encoder_acc = true

    -- encoder handling
    if #encoder_commands ~= 7 then
        LogLevel(LogLevels.ERR, "Invalid number of encoder commands... " .. #encoder_commands)
        EmergencyShutdown()
    end
    for k, v in pairs(encoder_commands) do
        if GetPruInterface():GetEncoderChanged(k - 1) then
            local val = GetPruInterface():GetEncoderValue(k - 1)
            --Log("About to call encoder command with val: " .. val)

            if return_home_time ~= nil then
                return_home_time = SDL.GetTicks()
            end

            -- ignore encoders for xxx ms when coming out of standby
            if ((SDL.GetTicks() - last_standby_time) > 2000) and not GetPreampSetting():GetStandby() then
                -- convert raw encoder ticks into 5 'buckets'...
                -- encoder ticks are not used directly, the values used come out of these buckets
                local abs_val = math.abs(val)
                local speed = 0
                if abs_val > 180 then
                    --Log("Max speed")
                    speed = 5.0
                elseif abs_val > 140 then
                    --Log("Speed 4")
                    speed = 4.0
                elseif abs_val > 100 then
                    --Log("Speed 3")
                    speed = 3.0
                elseif abs_val > 60 then
                    --Log("Speed 2")
                    speed = 2.0
                else
                    --Log("Speed 1")
                    if disable_encoder_acc == true then
                        speed = 1.0
                    else
                        speed = 0.5
                    end
                end
                --if val == 0 then
                --    speed = 0
                if val < 0 then
                    speed = -speed
                end
                encoder_acc[k] = encoder_acc[k] + speed
                local integer = math.modf(encoder_acc[k])
                if(integer ~= 0) then
                    --Log("Got enough ACC: " .. encoder_acc[k])
                    encoder_commands[k](k, integer)
                    encoder_acc[k] = 0
                else
                    --Log("Not enough ACC yet..." .. encoder_acc[k])
                end
            else
                Log("Just out of STANDBY, NOT changing encoders!")
            end
        end
    end

    -- remote handling
    if GetPruInterface():GetRemoteChanged() then
        local val = GetPruInterface():GetRemoteValue()
        Log("Got remote command: " .. val)
        if val ~= 0 then
            if remote_commands[val] then
                remote_commands[val]()
            else
                Log("No remote command associated with code: " .. val)
            end
        end
    end


    local config_ticks = SDL.GetTicks()
    if ((config_ticks - GetPreampSetting():GetConfigTimer()) > 1000) and last_config_time ~= GetPreampSetting():GetConfigTimer() then
        GetPreampSetting():SaveSettings()
        last_config_time = GetPreampSetting():GetConfigTimer()
    end

    local home_ticks = SDL.GetTicks()
    if return_home_time ~= nil and ((home_ticks - return_home_time) > 30000) then
        Log("Return to home timer expired")

        ClearPopupAndIcons()

        return_home_time = nil
    end
    
    if GetPreampSetting():GetUpdateInProgress() then
        local percent = GetPreampSetting():GetUpdatePercent()
        --TODO figure out how/where to print the update percent(and maybe update status)
        SingleLineText(percent, 6, 960, 4, 200, 20, 0, "center", 0, false, false)
    end
    --TODO print progress for update if going

    ReleaseDisplayManagerLock(dm)
    ReleasePreampSettingLock(preamp)
    ReleaseConfigLock(config)

    local end_ticks = SDL.GetTicks()
    --Log("draw time: " .. end_ticks - start_ticks)
    if end_ticks - start_ticks > 150 then
        SDL.Delay(1)
    else
        SDL.Delay(150 - (end_ticks - start_ticks))
    end
end
