
function CmdRemoteVolumeUp()
    local old = GetPreampSetting():GetMasterVolume()
    local new = AdjustHelper("CmdRemoteVolumeUp", old, 1, GetPreampSetting():GetVolumeResolution(),
                             GetPreampSetting():GetVolumeLimits())
    if old == -1001 then
        new = -1000
    end
    GetPreampSetting():SetMasterVolume(new)
end

function CmdRemoteVolumeDown()
    local old = GetPreampSetting():GetMasterVolume()
    local new = AdjustHelper("CmdRemoteVolumeDown", old, -1, GetPreampSetting():GetVolumeResolution(),
                             GetPreampSetting():GetVolumeLimits())
    GetPreampSetting():SetMasterVolume(new)
end

function CmdRemoteMute()
    local now = SDL.GetTicks()
    if now - last_mute_time > 500 then
        CmdMute(-1)
        last_mute_time = now
    end
end

function CmdRemoteSoftMute()
    local now = SDL.GetTicks()
    if now - last_mute_time > 500 then
        CmdSoft(-1)
        last_mute_time = now
    end
end

function CmdRemoteBalanceRight()
    if CheckStickyTimer() then
        return
    end
    local old = GetPreampSetting():GetBalance()
    local new = AdjustHelper("CmdRemoteBalanceRight", old, 1, GetPreampSetting():GetVolumeResolution(),
                             GetPreampSetting():GetBalanceLimits())
    if (old == -201) then
        new = -200
    end
    if SetStickyTimer(old, new) then
        new = 0
    end
    GetPreampSetting():SetBalance(new)
end

function CmdRemoteBalanceLeft()
    if CheckStickyTimer() then
        return
    end
    local old = GetPreampSetting():GetBalance()
    local new = AdjustHelper("CmdRemoteBalanceLeft", old, -1, GetPreampSetting():GetVolumeResolution(),
                             GetPreampSetting():GetBalanceLimits())
    if (old == 201) then
        new = 200
    end
    if SetStickyTimer(old, new) then
        new = 0
    end
    GetPreampSetting():SetBalance(new)
    
end

function CmdRemoteBlendDown()
    local old = GetPreampSetting():GetBlend()
    local new = AdjustHelper("CmdRemoteBlendDown", old, 1, 1,
                             GetPreampSetting():GetBlendLimits())
    GetPreampSetting():SetBlend(new)
end

function CmdRemoteBlendUp()
    local old = GetPreampSetting():GetBlend()
    local new = AdjustHelper("CmdRemoteBlendUp", old, -1, 1,
                             GetPreampSetting():GetBlendLimits())
    GetPreampSetting():SetBlend(new)
end

function CmdRemoteStandby()
    -- reuse the sticky timer to prevent repeated standbys... hacky?
    if CheckStickyTimer() then
        return
    end
    local old = GetPreampSetting():GetStandby()
    -- can't call real SetStickyTimer with a bool...
    command_sticky_time = SDL.GetTicks()
    if old then
        CmdPowerOnSpecial()
    else
        CmdPowerOff(99)
    end
end

function CmdRemoteBrightness()
    local old = GetPreampSetting():GetDisplayBrightness()
    local new = 0
    if old == 100 then
        new = 0
    else
        new = old + 5
    end
    Log("CmdRemoteBrightness: " .. new)
    GetPreampSetting():SetDisplayBrightness(new)
end

function CmdRemotePolarity()
    local now = SDL.GetTicks()
    if now - last_mute_time > 500 then
        local old = GetPreampSetting():GetPolarity()
        local new = not old
        Log("CmdRemotePolarity")
        GetPreampSetting():SetPolarity(new)
        last_mute_time = now
    end
end

function CmdRemoteRightPolarity()
    local now = SDL.GetTicks()
    if now - last_mute_time > 500 then
        local old = GetPreampSetting():GetPolarityRight()
        local new = not old
        Log("CmdRemoteRightPolarity")
        GetPreampSetting():SetPolarityRight(new)
        last_mute_time = now
    end
end

function CmdRemoteInputTrimDown()
    local input = GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput())
    local old = input.trim
    local new = AdjustHelper("CmdRemoteInputTrimDown", old, -1, 1, GetPreampSetting():GetInputLimits())
    GetPreampSetting():SetInputDevice(input.index, input.name, new, input.balance, input.polarity,
                                      input.theater_mode, input.recordable)
end

function CmdRemoteInputTrimUp()
    local input = GetPreampSetting():GetInputDevice(GetPreampSetting():GetCurrentInput())
    local old = input.trim
    local new = AdjustHelper("CmdRemoteInputTrimUp", old, 1, 1, GetPreampSetting():GetInputLimits())
    GetPreampSetting():SetInputDevice(input.index, input.name, new, input.balance, input.polarity,
                                      input.theater_mode, input.recordable)
end

function CmdRemoteOutput1Down()
    local output = GetPreampSetting():GetOutputDevice(0)
    local old = output.volume
    local new = AdjustHelper("CmdRemoteOutput1Down", old, -1, 5, GetPreampSetting():GetOutputLimits())
    GetPreampSetting():SetOutputDevice(output.name, 0, new, output.mute, output.polarity, output.polarity_temp)
end

function CmdRemoteOutput1Up()
    local output = GetPreampSetting():GetOutputDevice(0)
    local old = output.volume
    local new = AdjustHelper("CmdRemoteOutput1Up", old, 1, 5, GetPreampSetting():GetOutputLimits())
    GetPreampSetting():SetOutputDevice(output.name, 0, new, output.mute, output.polarity, output.polarity_temp)
end

function CmdRemoteOutput2Down()
    local output = GetPreampSetting():GetOutputDevice(1)
    local old = output.volume
    local new = AdjustHelper("CmdRemoteOutput2Down", old, -1, 5, GetPreampSetting():GetOutputLimits())
    GetPreampSetting():SetOutputDevice(output.name, 1, new, output.mute, output.polarity, output.polarity_temp)
end

function CmdRemoteOutput2Up()
    local output = GetPreampSetting():GetOutputDevice(1)
    local old = output.volume
    local new = AdjustHelper("CmdRemoteOutput2Up", old, 1, 5, GetPreampSetting():GetOutputLimits())
    GetPreampSetting():SetOutputDevice(output.name, 1, new, output.mute, output.polarity, output.polarity_temp)
end

function CmdRemoteOutput3Down()
    local output = GetPreampSetting():GetOutputDevice(2)
    local old = output.volume
    local new = AdjustHelper("CmdRemoteOutput3Down", old, -1, 5, GetPreampSetting():GetOutputLimits())
    GetPreampSetting():SetOutputDevice(output.name, 2, new, output.mute, output.polarity, output.polarity_temp)
end

function CmdRemoteOutput3Up()
    local output = GetPreampSetting():GetOutputDevice(2)
    local old = output.volume
    local new = AdjustHelper("CmdRemoteOutput3Up", old, 1, 5, GetPreampSetting():GetOutputLimits())
    GetPreampSetting():SetOutputDevice(output.name, 2, new, output.mute, output.polarity, output.polarity_temp)
end

function CmdRemoteOutput1Mute()
    local now = SDL.GetTicks()
    if now - last_mute_time > 500 then
        -- add offset of 9 to match the button number
        CmdOutputMute(1 + 9)
        last_mute_time = now
    end
end

function CmdRemoteOutput2Mute()
    local now = SDL.GetTicks()
    if now - last_mute_time > 500 then
        -- add offset of 9 to match the button number
        CmdOutputMute(2 + 9)
        last_mute_time = now
    end
end

function CmdRemoteOutput3Mute()
    local now = SDL.GetTicks()
    if now - last_mute_time > 500 then
        -- add offset of 9 to match the button number
        CmdOutputMute(3 + 9)
        last_mute_time = now
    end
end

function CmdRemoteInput1()
    CmdSwitchInput(1)
end

function CmdRemoteInput2()
    CmdSwitchInput(2)
end

function CmdRemoteInput3()
    CmdSwitchInput(3)
end

function CmdRemoteInput4()
    CmdSwitchInput(4)
end

function CmdRemoteInput5()
    CmdSwitchInput(5)
end

function CmdRemoteInput6()
    CmdSwitchInput(6)
end
