import flask
from flask import Flask, render_template, Blueprint, url_for, current_app, redirect, request, jsonify
import python_settings
import requests
from requests.exceptions import RequestException
import threading
from flask_bootstrap import Bootstrap

import libpreamp_lib as amp
from pylogger import bllogger
import python_settings
from preampsetting import LockCFunction
main = Blueprint('main', __name__ )


app = Flask(__name__)
Bootstrap(app)

@LockCFunction
def get_limits():
    limit={}
    res = {}
    limit['output'] = python_settings.preamp_setting.preamp_setting.GetOutputLimits()
    output_res = python_settings.preamp_setting.preamp_setting.GetOutputResolution()
    res['output'] = output_res
    limit['input'] = python_settings.preamp_setting.preamp_setting.GetInputLimits()
    res['input'] = python_settings.preamp_setting.preamp_setting.GetInputResolution()

    limit['input_balance'] = python_settings.preamp_setting.preamp_setting.GetInputBalanceLimits()

    limit['volume'] = python_settings.preamp_setting.preamp_setting.GetVolumeLimits()
    limit['volume'].first = python_settings.preamp_setting.preamp_setting.ScaleMasterVolume(limit['volume'].first)
    limit['volume'].second = python_settings.preamp_setting.preamp_setting.ScaleMasterVolume(limit['volume'].second)
    res['volume'] = python_settings.preamp_setting.preamp_setting.GetVolumeResolution()

    limit['balance'] = python_settings.preamp_setting.preamp_setting.GetBalanceLimits()

    limit['blend'] = python_settings.preamp_setting.preamp_setting.GetBlendLimits()

    limit['soft'] = python_settings.preamp_setting.preamp_setting.GetSoftLimits()
    res['soft'] = python_settings.preamp_setting.preamp_setting.GetSoftResolution()

    limit['default_vol'] = amp.VolumeLimit()
    limit['default_vol'].first = -900
    limit['default_vol'].second = -500

    limit['max_vol'] = amp.VolumeLimit()
    limit['max_vol'].first = -1000
    limit['max_vol'].second = 0

    return limit, res


def get_api_data():
    """
    api_endpoints = [
            {'url': 'api/room/room_name', 'name': 'room_name'},
            {'url': 'api/balance/balance', 'name': 'balance'},
            {'url': 'api/blend/blend', 'name': 'blend'},
            {'url': 'api/master_volume/master_volume', 'name': 'master_volume'},
            {'url': 'api/master_volume/default_volume', 'name': 'default_volume'},
            {'url': 'api/master_volume/max_volume', 'name': 'max_volume'},
            {'url': 'api/master_volume/volume_resolution', 'name': 'volume_resolution'},
            {'url': 'api/input/input_device', 'name': 'inputs'},
            {'url': 'api/input/current_input_device', 'name': 'current_input'},
            {'url': 'api/input/current_aux_device', 'name': 'current_aux'},
            {'url': 'api/input/aux_disabled', 'name': 'aux_disabled'},
            {'url': 'api/output/output_device', 'name': 'output_device'},
            {'url': 'api/version/version', 'name': 'software_version'},
            {'url': 'api/update/update_available', 'name': 'update_available'},
            {'url': 'api/update/update_in_progress', 'name': 'update_in_progress'},
            {'url': 'api/update/update_progress', 'name': 'update_progress'},
            {'url': 'api/soft/soft', 'name': 'soft'},
            {'url': 'api/soft/soft_level', 'name': 'soft_level'},
            {'url': 'api/mute/mute', 'name': 'mute'},
            {'url': 'api/polarity/polarity', 'name': 'polarity'},
            {'url': 'api/polarity/polarity_right', 'name': 'polarity_right'},
            {'url': 'api/polarity/polarity_aux', 'name': 'polarity_aux'},
            {'url': 'api/aux/aux_follows_main', 'name': 'aux_follows_main'},
            {'url': 'api/display/display_brightness', 'name': 'display_brightness'}
            ]""" 
    api_endpoints = [
            {'url': 'api/all/all', 'name': 'all'}
            ]
    responses = {}
    for endpoint in api_endpoints:
        try:
            r = requests.get(request.url_root+endpoint['url'], timeout=1)
            responses[endpoint['name']] = r.json()[endpoint['name']]
        except Exception as e:
            bllogger('err', "got exception when connecting to %s api (%s)" % (endpoint['name'], endpoint['url']))
            bllogger('err',  "%s" % e )
            responses[endpoint['name']] = ''

    return responses['all']


@app.route('/')
def main_display():
    print "###Getting endpoints###"

    responses = get_api_data()
    print responses
    limit, res = get_limits()
    return render_template('base.html',
                            responses=responses,
                            upload_link = url_for('.upload_logs'),
                            standby_link = url_for('.standby'),
                            update_link = url_for('.start_update'),
                            limit=limit,
                            res=res
			    )


@app.route('/ajax/all_data', methods=['GET'])
def ajax_get_data():
    responses = get_api_data()
    return jsonify({'responses': responses})

@app.route('/ajax', methods=['POST'])
def room_name_post():
    print 'values: ', request.values
    print 'args: ', request.args
    print 'form: ', request.form
    if 'room_name_form' in request.form:
	data = {'room_name': request.form['room_name']}
	url = 'api/room/room_name'
    elif 'balance_form' in request.form:
	data = {'balance': request.form['balance']}
	url = 'api/balance/balance'
    elif 'blend_form' in request.form:
	data = {'blend': request.form['blend']}
	url = 'api/blend/blend'
    elif 'master_volume_form' in request.form:
	data = {'master_volume': request.form['master_volume']}
	url = 'api/master_volume/master_volume'
    elif 'default_volume_form' in request.form:
	data = {'default_volume': request.form['default_volume']}
	url = 'api/master_volume/default_volume'
    elif 'max_volume_form' in request.form:
	data = {'max_volume': request.form['max_volume']}
	url = 'api/master_volume/max_volume'
    elif 'soft_level_form' in request.form:
	data = {'soft_level': request.form['soft_level']}
	url = 'api/soft/soft_level'
    elif 'current_aux_form' in request.form:
        if int(request.form['index']) < 0:
            data = {'aux_disabled': 'true'}
            url = 'api/input/aux_disabled'
        else:
            data = {'index': request.form['index']}
            url = 'api/input/current_aux_device'
    elif 'current_input_form' in request.form:
        if int(request.form['index']) < 0:
            data = {'input_disabled': 'true'}
            url = 'api/input/input_disabled'
        else:
            data = {'index': request.form['index']}
            url = 'api/input/current_input_device'
    elif 'input_form' in request.form:
	data = {'index': request.form['index'],
		'trim': request.form['trim'],
	    }
	url = 'api/input/input_device'
    elif 'setting_input_form' in request.form:
	data = {'index': request.form['index'],
                'name': request.form['name'],
                'balance': request.form['input_balance']
	    }
        data['polarity'] = 'true' if 'polarity' in request.form else 'false'
        data['theater_mode'] = 'true' if 'theater_mode' in request.form else 'false'
        data['recordable'] = 'true' if 'recordable' in request.form else 'false'

	url = 'api/input/input_device'
    elif 'output_form' in request.form:
	data = {'index': request.form['index'],
		'volume': request.form['output_volume'],
	    }
        data['mute'] = 'true' if 'mute' in request.form else 'false'
	url = 'api/output/output_device'
    elif 'output_polarity_temp_form' in request.form:
	data = {'index': request.form['index'],
            }
        data['polarity_temp'] = 'true' if 'polarity_temp' in request.form else 'false'
	url = 'api/output/output_device'
    elif 'setting_output_form' in request.form:
	data = {'index': request.form['index'],
		'name': request.form['name'],
            }
        data['polarity'] = 'true' if 'polarity' in request.form else 'false'
	url = 'api/output/output_device'
    elif 'soft_mute_form' in request.form:
        data = {}
        muted = python_settings.preamp_setting.preamp_setting.GetSoft()
        data['soft'] = 'false' if muted  else 'true'
	url = 'api/soft/soft'
    elif 'mute_form' in request.form:
        data = {}
        muted = python_settings.preamp_setting.preamp_setting.GetMute()
        print "muted:  ", muted, " request: ", request.form
        data['mute'] = 'false' if muted  else 'true'
        print "data: ", data
	url = 'api/mute/mute'
    elif 'polarity_absolute_form' in request.form:
        data = {}
        polarity = python_settings.preamp_setting.preamp_setting.GetPolarity()
        data['polarity'] = 'false' if polarity  else 'true'
	url = 'api/polarity/polarity'
    elif 'polarity_right_form' in request.form:
        data = {}
        polarity = python_settings.preamp_setting.preamp_setting.GetPolarityRight()
        data['polarity'] = 'false' if polarity  else 'true'
	url = 'api/polarity/polarity_right'
    elif 'aux_polarity_form' in request.form:
        data = {}
        polarity = python_settings.preamp_setting.preamp_setting.GetPolarityAux()
        data['polarity_aux'] = 'false' if polarity  else 'true'
	url = 'api/polarity/polarity_aux'
    elif 'aux_disabled_form' in request.form:
        data = {}
        aux_disabled = python_settings.preamp_setting.preamp_setting.GetAuxDisabled()
        data['aux_disabled'] = 'false' if aux_disabled  else 'true'
	url = 'api/input/aux_disabled'
    elif 'aux_follows_main_form' in request.form:
        data = {}
        follows = python_settings.preamp_setting.preamp_setting.GetAuxDisabled()
        data['follows'] = 'false' if follows  else 'true'
	url = 'api/aux/aux_follows_main'
    elif 'resolution_form' in request.form:
        data = {'volume_resolution': request.form['resolution']}
	url = 'api/master_volume/volume_resolution'
    elif 'display_brightness_form' in request.form:
        data = {'display_brightness': request.form['display_brightness']}
	url = 'api/display/display_brightness'
    elif 'reset_input_names_form' in request.form:
        data = {}
        url = 'api/input/reset_input_names'
    elif 'reset_output_names_form' in request.form:
        data = {}
        url = 'api/input/reset_output_names'
    elif 'restore_factory_defaults_form' in request.form:
        data = {}
        url = 'api/global/restore_factory_defaults'
    else:
	bllogger('err', 'Form type not reconized')
        return jsonify({'form': request.form})
    try:
        print 'data: ', data
        r = requests.post(request.url_root+url, params=data, timeout=1)
    except Exception as e:
	bllogger('err', 'got an exception when connecting to %s api' % url)
        bllogger('err', "%s" % e)
    if r.status_code is not 200:
	bllogger('err', '%s return code %s' % (url, r.status_code))

    return jsonify({'form': request.form})


@app.route('/debug')
def swagger():
    return render_template('index.html',
                            base_url = request.url_root)


@app.route('/other/standby')
def standby():
    try:
        r = requests.put(request.url_root+'api/standby/standby', timeout=1)
    except Exception as e:
	bllogger('err', 'got an exception when connecting to the standby api')
        bllogger('err', "%s" % e)
    return redirect(url_for('.main_display'))

@app.route('/log/upload')
def upload_logs():
    try:
        r = requests.put(request.url_root+'api/log/upload', timeout=1)
    except Exception as e:
        print "got exception when connecting to the upload_log api"
        print e
    return redirect(url_for('.main_display'))


@app.route('/update/start')
def start_update():
    try:
        r = requests.put(request.url_root+'api/update/update_available', timeout=1)
    except Exception as e:
        print "got exception when connecting to the update_avialable api"
        print e
    return redirect(url_for('.main_display'))
